@@ -35,20 +35,6 @@ typedef enum {
 
 extern struct divecomputer *select_dc(struct dive *);
 
-struct print_options {
-	enum print_type {
-		PRETTY,
-		TABLE,
-		TWOPERPAGE,
-		ONEPERPAGE
-	} type;
-	bool print_selected;
-	bool color_selected;
-	bool notes_up;
-	bool landscape;
-	int margins[4]; // left, top, right, bottom
-};
-
 extern unsigned int dc_number;
 
 extern unsigned int amount_selected;
@@ -67,7 +53,7 @@ int enumerate_devices(device_callback_t callback, void *userdata, int dc_type);
 extern const char *default_dive_computer_vendor;
 extern const char *default_dive_computer_product;
 extern const char *default_dive_computer_device;
-
+extern int default_dive_computer_download_mode;
 #define AMB_PERCENTAGE 50.0
 
 #ifdef __cplusplus
