@@ -8,11 +8,14 @@
 #include "dive.h"
 #include "libdivecomputer.h"
 #include "device.h"
+#include "divelist.h"
+#include "qthelperfromc.h"
 
 /* one could argue about the best place to have this variable -
  * it's used in the UI, but it seems to make the most sense to have it
  * here */
 struct dive displayed_dive;
+struct dive_site displayed_dive_site;
 
 struct tag_entry *g_tag_list = NULL;
 
@@ -391,6 +394,7 @@ static void copy_pl(struct picture *sp, struct picture *dp)
 {
 	*dp = *sp;
 	dp->filename = copy_string(sp->filename);
+	dp->hash = copy_string(sp->hash);
 }
 
 /* copy an element in a list of tags */
@@ -437,7 +441,6 @@ void clear_dive(struct dive *d)
 	/* free the strings */
 	free(d->buddy);
 	free(d->divemaster);
-	free(d->location);
 	free(d->notes);
 	free(d->suit);
 	/* free tags, additional dive computers, and pictures */
@@ -463,7 +466,6 @@ void copy_dive(struct dive *s, struct dive *d)
 	*d = *s;
 	d->buddy = copy_string(s->buddy);
 	d->divemaster = copy_string(s->divemaster);
-	d->location = copy_string(s->location);
 	d->notes = copy_string(s->notes);
 	d->suit = copy_string(s->suit);
 	for (int i = 0; i < MAX_CYLINDERS; i++)
@@ -500,7 +502,6 @@ void selective_copy_dive(struct dive *s, struct dive *d, struct dive_components
 {
 	if (clear)
 		clear_dive(d);
-	CONDITIONAL_COPY_STRING(location);
 	CONDITIONAL_COPY_STRING(notes);
 	CONDITIONAL_COPY_STRING(divemaster);
 	CONDITIONAL_COPY_STRING(buddy);
@@ -509,10 +510,8 @@ void selective_copy_dive(struct dive *s, struct dive *d, struct dive_components
 		d->rating = s->rating;
 	if (what.visibility)
 		d->visibility = s->visibility;
-	if (what.gps) {
-		d->longitude = s->longitude;
-		d->latitude = s->latitude;
-	}
+	if (what.divesite)
+		d->dive_site_uuid = s->dive_site_uuid;
 	if (what.tags)
 		STRUCTURED_LIST_COPY(struct tag_entry, s->tag_list, d->tag_list, copy_tl);
 	if (what.cylinders)
@@ -730,6 +729,8 @@ void per_cylinder_mean_depth(struct dive *dive, struct divecomputer *dc, int *me
 
 	for (i = 0; i < MAX_CYLINDERS; i++)
 		mean[i] = duration[i] = 0;
+	if (!dc)
+		return;
 	struct event *ev = get_next_event(dc->events, "gaschange");
 	if (!ev || (dc && dc->sample && ev->time.seconds == dc->sample[0].time.seconds && get_next_event(ev->next, "gaschange") == NULL)) {
 		// we have either no gas change or only one gas change and that's setting an explicit first cylinder
@@ -799,6 +800,19 @@ static void fixup_pressure(struct dive *dive, struct sample *sample, enum cylind
 	if (!pressure)
 		return;
 
+	/*
+	 * Ignore surface samples for tank pressure information.
+	 *
+	 * At the beginning of the dive, let the cylinder cool down
+	 * if the diver starts off at the surface. And at the end
+	 * of the dive, there may be surface pressures where the
+	 * diver has already turned off the air supply (especially
+	 * for computers like the Uemis Zurich that end up saving
+	 * quite a bit of samples after the dive has ended).
+	 */
+	if (sample->depth.mm < SURFACE_THRESHOLD)
+		return;
+
 	/* FIXME! sensor -> cylinder mapping? */
 	if (index >= MAX_CYLINDERS)
 		return;
@@ -863,13 +877,14 @@ static int same_rounded_pressure(pressure_t a, pressure_t b)
  * first cylinder - in which case cylinder 0 is indeed the first cylinder */
 int explicit_first_cylinder(struct dive *dive, struct divecomputer *dc)
 {
-	struct event *ev = get_next_event(dc->events, "gaschange");
-	if (ev && dc && dc->sample && ev->time.seconds == dc->sample[0].time.seconds)
-		return get_cylinder_index(dive, ev);
-	else if (dc->divemode == CCR)
-		return MAX(get_cylinder_idx_by_use(dive, DILUENT), 0);
-	else
-		return 0;
+	if (dc) {
+		struct event *ev = get_next_event(dc->events, "gaschange");
+		if (ev && dc->sample && ev->time.seconds == dc->sample[0].time.seconds)
+			return get_cylinder_index(dive, ev);
+		else if (dc->divemode == CCR)
+			return MAX(get_cylinder_idx_by_use(dive, DILUENT), 0);
+	}
+	return 0;
 }
 
 /* this gets called when the dive mode has changed (so OC vs. CC)
@@ -1054,10 +1069,10 @@ static struct event *find_previous_event(struct divecomputer *dc, struct event *
 	struct event *ev = dc->events;
 	struct event *previous = NULL;
 
-	if (!event->name)
+	if (same_string(event->name, ""))
 		return NULL;
 	while (ev && ev != event) {
-		if (ev->name && !strcmp(ev->name, event->name))
+		if (same_string(ev->name, event->name))
 			previous = ev;
 		ev = ev->next;
 	}
@@ -1367,6 +1382,13 @@ struct dive *fixup_dive(struct dive *dive)
 	sanitize_cylinder_info(dive);
 	dive->maxcns = dive->cns;
 
+	/*
+	 * Use the dive's temperatures for minimum and maximum in case
+	 * we do not have temperatures recorded by DC.
+	 */
+
+	update_min_max_temperatures(dive, dive->watertemp);
+
 	for_each_dc (dive, dc)
 		fixup_dive_dc(dive, dc);
 
@@ -1385,6 +1407,7 @@ struct dive *fixup_dive(struct dive *dive)
 		if (same_rounded_pressure(cyl->sample_end, cyl->end))
 			cyl->end.mbar = 0;
 	}
+	update_cylinder_related_info(dive);
 	for (i = 0; i < MAX_WEIGHTSYSTEMS; i++) {
 		weightsystem_t *ws = dive->weightsystem + i;
 		add_weightsystem_description(ws);
@@ -1403,7 +1426,7 @@ struct dive *fixup_dive(struct dive *dive)
 #define MERGE_TXT(res, a, b, n) res->n = merge_text(a->n, b->n)
 #define MERGE_NONZERO(res, a, b, n) res->n = a->n ? a->n : b->n
 
-static struct sample *add_sample(struct sample *sample, int time, struct divecomputer *dc)
+struct sample *add_sample(struct sample *sample, int time, struct divecomputer *dc)
 {
 	struct sample *p = prepare_sample(dc);
 
@@ -1761,7 +1784,7 @@ static void add_initial_gaschange(struct dive *dive, struct divecomputer *dc)
 	add_gas_switch_event(dive, dc, 0, 0);
 }
 
-static void dc_cylinder_renumber(struct dive *dive, struct divecomputer *dc, int mapping[])
+void dc_cylinder_renumber(struct dive *dive, struct divecomputer *dc, int mapping[])
 {
 	int i;
 	struct event *ev;
@@ -2214,6 +2237,11 @@ static int likely_same_dive(struct dive *a, struct dive *b)
 {
 	int match, fuzz = 20 * 60;
 
+	/* don't merge manually added dives with anything */
+	if (same_string(a->dc.model, "manually added dive") ||
+	    same_string(b->dc.model, "manually added dive"))
+		return 0;
+
 	/* Don't try to merge dives with different trip information */
 	if (a->divetrip != b->divetrip) {
 		/*
@@ -2262,7 +2290,7 @@ struct dive *try_to_merge(struct dive *a, struct dive *b, bool prefer_downloaded
 	return NULL;
 }
 
-static void free_events(struct event *ev)
+void free_events(struct event *ev)
 {
 	while (ev) {
 		struct event *next = ev->next;
@@ -2721,7 +2749,7 @@ int count_dives_with_location(const char *location)
 	struct dive *d;
 
 	for_each_dive (i, d) {
-		if (same_string(d->location, location))
+		if (same_string(get_dive_location(d), location))
 			counter++;
 	}
 	return counter;
@@ -2740,35 +2768,52 @@ int count_dives_with_suit(const char *suit)
 	return counter;
 }
 
-
+/*
+ * Merging two dives can be subtle, because there's two different ways
+ * of merging:
+ *
+ * (a) two distinctly _different_ dives that have the same dive computer
+ *     are merged into one longer dive, because the user asked for it
+ *     in the divelist.
+ *
+ *     Because this case is with teh same dive computer, we *know* the
+ *     two must have a different start time, and "offset" is the relative
+ *     time difference between the two.
+ *
+ * (a) two different dive computers that we migth want to merge into
+ *     one single dive with multiple dive computers.
+ *
+ *     This is the "try_to_merge()" case, which will have offset == 0,
+ *     even if the dive times migth be different.
+ */
 struct dive *merge_dives(struct dive *a, struct dive *b, int offset, bool prefer_downloaded)
 {
 	struct dive *res = alloc_dive();
 	struct dive *dl = NULL;
 
-	/* Aim for newly downloaded dives to be 'b' (keep old dive data first) */
-	if (a->downloaded && !b->downloaded) {
-		struct dive *tmp = a;
-		a = b;
-		b = tmp;
+	if (offset) {
+		/*
+		 * If "likely_same_dive()" returns true, that means that
+		 * it is *not* the same dive computer, and we do not want
+		 * to try to turn it into a single longer dive. So we'd
+		 * join them as two separate dive computers at zero offset.
+		 */
+		if (likely_same_dive(a, b))
+			offset = 0;
+	} else {
+		/* Aim for newly downloaded dives to be 'b' (keep old dive data first) */
+		if (a->downloaded && !b->downloaded) {
+			struct dive *tmp = a;
+			a = b;
+			b = tmp;
+		}
+		if (prefer_downloaded && b->downloaded)
+			dl = b;
 	}
-	if (prefer_downloaded && b->downloaded)
-		dl = b;
-
-	/*
-	 * Did the user ask us to merge dives in the dive list?
-	 * We may want to just join the dive computers, not try to
-	 * interleave them at some offset.
-	 */
-	if (offset && likely_same_dive(a, b))
-		offset = 0;
 
 	res->when = dl ? dl->when : a->when;
 	res->selected = a->selected || b->selected;
 	merge_trip(res, a, b);
-	MERGE_NONZERO(res, a, b, latitude.udeg);
-	MERGE_NONZERO(res, a, b, longitude.udeg);
-	MERGE_TXT(res, a, b, location);
 	MERGE_TXT(res, a, b, notes);
 	MERGE_TXT(res, a, b, buddy);
 	MERGE_TXT(res, a, b, divemaster);
@@ -2788,11 +2833,267 @@ struct dive *merge_dives(struct dive *a, struct dive *b, int offset, bool prefer
 		interleave_dive_computers(&res->dc, &a->dc, &b->dc, offset);
 	else
 		join_dive_computers(&res->dc, &a->dc, &b->dc, 0);
-
+	res->dive_site_uuid = a->dive_site_uuid ?: b->dive_site_uuid;
 	fixup_dive(res);
 	return res;
 }
 
+// copy_dive(), but retaining the new ID for the copied dive
+static struct dive *create_new_copy(struct dive *from)
+{
+	struct dive *to = alloc_dive();
+	int id;
+
+	// alloc_dive() gave us a new ID, we just need to
+	// make sure it's not overwritten.
+	id = to->id;
+	copy_dive(from, to);
+	to->id = id;
+	return to;
+}
+
+static void force_fixup_dive(struct dive *d)
+{
+	struct divecomputer *dc = &d->dc;
+	int old_maxdepth = dc->maxdepth.mm;
+	int old_temp = dc->watertemp.mkelvin;
+	int old_mintemp = d->mintemp.mkelvin;
+	int old_maxtemp = d->maxtemp.mkelvin;
+	duration_t old_duration = d->duration;
+
+	d->maxdepth.mm = 0;
+	dc->maxdepth.mm = 0;
+	d->watertemp.mkelvin = 0;
+	dc->watertemp.mkelvin = 0;
+	d->duration.seconds = 0;
+	d->maxtemp.mkelvin = 0;
+	d->mintemp.mkelvin = 0;
+
+	fixup_dive(d);
+
+	if (!d->watertemp.mkelvin)
+		d->watertemp.mkelvin = old_temp;
+
+	if (!dc->watertemp.mkelvin)
+		dc->watertemp.mkelvin = old_temp;
+
+	if (!d->maxtemp.mkelvin)
+		d->maxtemp.mkelvin = old_maxtemp;
+
+	if (!d->mintemp.mkelvin)
+		d->mintemp.mkelvin = old_mintemp;
+
+	if (!d->duration.seconds)
+		d->duration = old_duration;
+
+}
+
+/*
+ * Split a dive that has a surface interval from samples 'a' to 'b'
+ * into two dives.
+ */
+static int split_dive_at(struct dive *dive, int a, int b)
+{
+	int i, t, nr;
+	struct dive *d1, *d2;
+	struct divecomputer *dc1, *dc2;
+	struct event *event, **evp;
+
+	/* if we can't find the dive in the dive list, don't bother */
+	if ((nr = get_divenr(dive)) < 0)
+		return 0;
+
+	/* We're not trying to be efficient here.. */
+	d1 = create_new_copy(dive);
+	d2 = create_new_copy(dive);
+
+	/* now unselect the first first segment so we don't keep all
+	 * dives selected by mistake. But do keep the second one selected
+	 * so the algorithm keeps splitting the dive further */
+	d1->selected = false;
+
+	dc1 = &d1->dc;
+	dc2 = &d2->dc;
+	/*
+	 * Cut off the samples of d1 at the beginning
+	 * of the interval.
+	 */
+	dc1->samples = a;
+
+	/* And get rid of the 'b' first samples of d2 */
+	dc2->samples -= b;
+	memmove(dc2->sample, dc2->sample+b, dc2->samples * sizeof(struct sample));
+
+	/*
+	 * This is where we cut off events from d1,
+	 * and shift everything in d2
+	 */
+	t = dc2->sample[0].time.seconds;
+	d2->when += t;
+	for (i = 0; i < dc2->samples; i++)
+		dc2->sample[i].time.seconds -= t;
+
+	/* Remove the events past 't' from d1 */
+	evp = &dc1->events;
+	while ((event = *evp) != NULL && event->time.seconds < t)
+		evp = &event->next;
+	*evp = NULL;
+	while (event) {
+		struct event *next = event->next;
+		free(event);
+		event = next;
+	}
+
+	/* Remove the events before 't' from d2, and shift the rest */
+	evp = &dc2->events;
+	while ((event = *evp) != NULL) {
+		if (event->time.seconds < t) {
+			*evp = event->next;
+			free(event);
+		} else {
+			event->time.seconds -= t;
+		}
+	}
+
+	force_fixup_dive(d1);
+	force_fixup_dive(d2);
+
+	if (dive->divetrip) {
+		d1->divetrip = d2->divetrip = 0;
+		add_dive_to_trip(d1, dive->divetrip);
+		add_dive_to_trip(d2, dive->divetrip);
+	}
+
+	delete_single_dive(nr);
+	add_single_dive(nr, d1);
+
+	/*
+	 * Was the dive numbered? If it was the last dive, then we'll
+	 * increment the dive number for the tail part that we split off.
+	 * Otherwise the tail is unnumbered.
+	 */
+	if (d2->number) {
+		if (dive_table.nr == nr + 1)
+			d2->number++;
+		else
+			d2->number = 0;
+	}
+	add_single_dive(nr + 1, d2);
+
+	mark_divelist_changed(true);
+
+	return 1;
+}
+
+/* in freedive mode we split for as little as 10 seconds on the surface,
+ * otherwise we use a minute */
+static bool should_split(struct divecomputer *dc, int t1, int t2)
+{
+	int threshold = dc->divemode == FREEDIVE ? 10 : 60;
+
+	return t2 - t1 >= threshold;
+}
+
+/*
+ * Try to split a dive into multiple dives at a surface interval point.
+ *
+ * NOTE! We will not split dives with multiple dive computers, and
+ * only split when there is at least one surface event that has
+ * non-surface events on both sides.
+ *
+ * In other words, this is a (simplified) reversal of the dive merging.
+ */
+int split_dive(struct dive *dive)
+{
+	int i;
+	int at_surface, surface_start;
+	struct divecomputer *dc;
+
+	if (!dive || (dc = &dive->dc)->next)
+		return 0;
+
+	surface_start = 0;
+	at_surface = 1;
+	for (i = 1; i < dc->samples; i++) {
+		struct sample *sample = dc->sample+i;
+		int surface_sample = sample->depth.mm < SURFACE_THRESHOLD;
+
+		/*
+		 * We care about the transition from and to depth 0,
+		 * not about the depth staying similar.
+		 */
+		if (at_surface == surface_sample)
+			continue;
+		at_surface = surface_sample;
+
+		// Did it become surface after having been non-surface? We found the start
+		if (at_surface) {
+			surface_start = i;
+			continue;
+		}
+
+		// Goind down again? We want at least a minute from
+		// the surface start.
+		if (!surface_start)
+			continue;
+		if (!should_split(dc, dc->sample[surface_start].time.seconds, sample[i - 1].time.seconds))
+			continue;
+
+		return split_dive_at(dive, surface_start, i-1);
+	}
+	return 0;
+}
+
+/*
+ * "dc_maxtime()" is how much total time this dive computer
+ * has for this dive. Note that it can differ from "duration"
+ * if there are surface events in the middle.
+ *
+ * Still, we do ignore all but the last surface samples from the
+ * end, because some divecomputers just generate lots of them.
+ */
+static inline int dc_totaltime(const struct divecomputer *dc)
+{
+	int time = dc->duration.seconds;
+	int nr = dc->samples;
+
+	while (nr--) {
+		struct sample *s = dc->sample + nr;
+		time = s->time.seconds;
+		if (s->depth.mm >= SURFACE_THRESHOLD)
+			break;
+	}
+	return time;
+}
+
+/*
+ * The end of a dive is actually not trivial, because "duration"
+ * is not the duration until the end, but the time we spend under
+ * water, which can be very different if there are surface events
+ * during the dive.
+ *
+ * So walk the dive computers, looking for the longest actual
+ * time in the samples (and just default to the dive duration if
+ * there are no samples).
+ */
+static inline int dive_totaltime(const struct dive *dive)
+{
+	int time =  dive->duration.seconds;
+	const struct divecomputer *dc;
+
+	for_each_dc(dive, dc) {
+		int dc_time = dc_totaltime(dc);
+		if (dc_time > time)
+			time = dc_time;
+	}
+	return time;
+}
+
+timestamp_t dive_endtime(const struct dive *dive)
+{
+	return dive->when + dive_totaltime(dive);
+}
+
 struct dive *find_dive_including(timestamp_t when)
 {
 	int i;
@@ -2802,15 +3103,24 @@ struct dive *find_dive_including(timestamp_t when)
 	 * also we always use the duration from the first divecomputer
 	 *     could this ever be a problem? */
 	for_each_dive (i, dive) {
-		if (dive->when <= when && when <= dive->when + dive->duration.seconds)
+		if (dive->when <= when && when <= dive_endtime(dive))
 			return dive;
 	}
 	return NULL;
 }
 
+bool time_during_dive_with_offset(struct dive *dive, timestamp_t when, timestamp_t offset)
+{
+	timestamp_t start = dive->when;
+	timestamp_t end = dive_endtime(dive);
+	return start - offset <= when && when <= end + offset;
+}
+
 bool dive_within_time_range(struct dive *dive, timestamp_t when, timestamp_t offset)
 {
-	return when - offset <= dive->when && dive->when + dive->duration.seconds <= when + offset;
+	timestamp_t start = dive->when;
+	timestamp_t end = dive_endtime(dive);
+	return when - offset <= start && end <= when + offset;
 }
 
 /* find the n-th dive that is part of a group of dives within the offset around 'when'.
@@ -2866,15 +3176,61 @@ void set_userid(char *rUserId)
 		prefs.userid[30]='\0';
 }
 
-int average_depth(struct diveplan *dive)
+/* this sets a usually unused copy of the preferences with the units
+ * that were active the last time the dive list was saved to git storage
+ * (this isn't used in XML files); storing the unit preferences in the
+ * data file is usually pointless (that's a setting of the software,
+ * not a property of the data), but it's a great hint of what the user
+ * might expect to see when creating a backend service that visualizes
+ * the dive list without Subsurface running - so this is basically a
+ * functionality for the core library that Subsurface itself doesn't
+ * use but that another consumer of the library (like an HTML exporter)
+ * will need */
+void set_informational_units(char *units)
+{
+	if (strstr(units, "METRIC")) {
+		informational_prefs.unit_system = METRIC;
+	} else if (strstr(units, "IMPERIAL")) {
+		informational_prefs.unit_system = IMPERIAL;
+	} else if (strstr(units, "PERSONALIZE")) {
+		informational_prefs.unit_system = PERSONALIZE;
+		if (strstr(units, "METERS"))
+			informational_prefs.units.length = METERS;
+		if (strstr(units, "FEET"))
+			informational_prefs.units.length = FEET;
+		if (strstr(units, "LITER"))
+			informational_prefs.units.volume = LITER;
+		if (strstr(units, "CUFT"))
+			informational_prefs.units.volume = CUFT;
+		if (strstr(units, "BAR"))
+			informational_prefs.units.pressure = BAR;
+		if (strstr(units, "PSI"))
+			informational_prefs.units.pressure = PSI;
+		if (strstr(units, "PASCAL"))
+			informational_prefs.units.pressure = PASCAL;
+		if (strstr(units, "CELSIUS"))
+			informational_prefs.units.temperature = CELSIUS;
+		if (strstr(units, "FAHRENHEIT"))
+			informational_prefs.units.temperature = FAHRENHEIT;
+		if (strstr(units, "KG"))
+			informational_prefs.units.weight = KG;
+		if (strstr(units, "LBS"))
+			informational_prefs.units.weight = LBS;
+		if (strstr(units, "SECONDS"))
+			informational_prefs.units.vertical_speed_time = SECONDS;
+		if (strstr(units, "MINUTES"))
+			informational_prefs.units.vertical_speed_time = MINUTES;
+	}
+}
+
+void average_max_depth(struct diveplan *dive, int *avg_depth, int *max_depth)
 {
 	int integral = 0;
 	int last_time = 0;
 	int last_depth = 0;
 	struct divedatapoint *dp = dive->dp;
 
-	if (!dp)
-		return 0;
+	*max_depth = 0;
 
 	while (dp) {
 		if (dp->time) {
@@ -2882,13 +3238,15 @@ int average_depth(struct diveplan *dive)
 			integral += (dp->depth + last_depth) * (dp->time - last_time) / 2;
 			last_time = dp->time;
 			last_depth = dp->depth;
+			if (dp->depth > *max_depth)
+				*max_depth = dp->depth;
 		}
 		dp = dp->next;
 	}
 	if (last_time)
-		return integral / last_time;
+		*avg_depth = integral / last_time;
 	else
-		return 0;
+		*avg_depth = *max_depth = 0;
 }
 
 struct picture *alloc_picture()
@@ -2912,69 +3270,99 @@ static bool new_picture_for_dive(struct dive *d, char *filename)
 // only add pictures that have timestamps between 30 minutes before the dive and
 // 30 minutes after the dive ends
 #define D30MIN (30 * 60)
-void dive_create_picture(struct dive *d, char *filename, int shift_time)
+bool dive_check_picture_time(struct dive *d, int shift_time, timestamp_t timestamp)
 {
-	timestamp_t timestamp;
-	if (!new_picture_for_dive(d, filename))
-		return;
-	struct picture *p = alloc_picture();
-	p->filename = filename;
-	picture_load_exif_data(p, &timestamp);
+	offset_t offset;
 	if (timestamp) {
-		p->offset.seconds = timestamp - d->when + shift_time;
-		if (p->offset.seconds < -D30MIN || p->offset.seconds > (int)d->duration.seconds + D30MIN) {
-			// this picture doesn't belong to this dive
-			free(p);
-			return;
+		offset.seconds = timestamp - d->when + shift_time;
+		if (offset.seconds > -D30MIN && offset.seconds < dive_totaltime(d) + D30MIN) {
+			// this picture belongs to this dive
+			return true;
 		}
 	}
-	dive_add_picture(d, p);
-	dive_set_geodata_from_picture(d, p);
+	return false;
 }
 
-void dive_add_picture(struct dive *d, struct picture *newpic)
+bool picture_check_valid(char *filename, int shift_time)
 {
-	struct picture **pic_ptr = &d->picture_list;
+	int i;
+	struct dive *dive;
+
+	timestamp_t timestamp = picture_get_timestamp(filename);
+	for_each_dive (i, dive)
+		if (dive->selected && dive_check_picture_time(dive, shift_time, timestamp))
+			return true;
+	return false;
+}
+
+void dive_create_picture(struct dive *dive, char *filename, int shift_time, bool match_all)
+{
+	timestamp_t timestamp = picture_get_timestamp(filename);
+	if (!new_picture_for_dive(dive, filename))
+		return;
+	if (!match_all && !dive_check_picture_time(dive, shift_time, timestamp))
+		return;
+
+	struct picture *picture = alloc_picture();
+	picture->filename = strdup(filename);
+	picture->offset.seconds = timestamp - dive->when + shift_time;
+	picture_load_exif_data(picture);
+
+	dive_add_picture(dive, picture);
+	dive_set_geodata_from_picture(dive, picture);
+}
+
+void dive_add_picture(struct dive *dive, struct picture *newpic)
+{
+	struct picture **pic_ptr = &dive->picture_list;
 	/* let's keep the list sorted by time */
 	while (*pic_ptr && (*pic_ptr)->offset.seconds <= newpic->offset.seconds)
 		pic_ptr = &(*pic_ptr)->next;
 	newpic->next = *pic_ptr;
 	*pic_ptr = newpic;
+	cache_picture(newpic);
 	return;
 }
 
-unsigned int dive_get_picture_count(struct dive *d)
+unsigned int dive_get_picture_count(struct dive *dive)
 {
 	unsigned int i = 0;
-	FOR_EACH_PICTURE (d)
+	FOR_EACH_PICTURE (dive)
 		i++;
 	return i;
 }
 
-void dive_set_geodata_from_picture(struct dive *d, struct picture *pic)
+void dive_set_geodata_from_picture(struct dive *dive, struct picture *picture)
 {
-	if (!d->latitude.udeg && pic->latitude.udeg) {
-		d->latitude = pic->latitude;
-		d->longitude = pic->longitude;
+	struct dive_site *ds = get_dive_site_by_uuid(dive->dive_site_uuid);
+	if (!dive_site_has_gps_location(ds) && (picture->latitude.udeg || picture->longitude.udeg)) {
+		if (ds) {
+			ds->latitude = picture->latitude;
+			ds->longitude = picture->longitude;
+		} else {
+			dive->dive_site_uuid = create_dive_site_with_gps("", picture->latitude, picture->longitude, dive->when);
+		}
 	}
 }
 
-static void picture_free(struct picture *p)
+static void picture_free(struct picture *picture)
 {
-	if (!p)
+	if (!picture)
 		return;
-	free(p->filename);
-	free(p);
+	free(picture->filename);
+	free(picture->hash);
+	free(picture);
 }
+
 void dive_remove_picture(char *filename)
 {
-	struct picture **ep = &current_dive->picture_list;
-	while (ep && !same_string((*ep)->filename, filename))
-		ep = &(*ep)->next;
-	if (ep) {
-		struct picture *temp = (*ep)->next;
-		picture_free(*ep);
-		*ep = temp;
+	struct picture **picture = &current_dive->picture_list;
+	while (picture && !same_string((*picture)->filename, filename))
+		picture = &(*picture)->next;
+	if (picture) {
+		struct picture *temp = (*picture)->next;
+		picture_free(*picture);
+		*picture = temp;
 	}
 }
 
@@ -3038,3 +3426,17 @@ void delete_current_divecomputer(void)
 	if (dc_number == count_divecomputers())
 		dc_number--;
 }
+
+/* helper function to make it easier to work with our structures
+ * we don't interpolate here, just use the value from the last sample up to that time */
+int get_depth_at_time(struct divecomputer *dc, int time)
+{
+	int depth = 0;
+	if (dc && dc->sample)
+		for (int i = 0; i < dc->samples; i++) {
+			if (dc->sample[i].time.seconds > time)
+				break;
+			depth = dc->sample[i].depth.mm;
+		}
+	return depth;
+}
