@@ -9,6 +9,9 @@
 #include "display.h"
 
 #include "libdivecomputer.h"
+#include <libdivecomputer/uwatec.h>
+#include <libdivecomputer/hw.h>
+
 
 /* Christ. Libdivecomputer has the worst configuration system ever. */
 #ifdef HW_FROG_H
@@ -26,13 +29,46 @@ double progress_bar_fraction = 0.0;
 static int stoptime, stopdepth, ndl, po2, cns;
 static bool in_deco, first_temp_is_air;
 
+/*
+ * Directly taken from libdivecomputer's examples/common.c to improve
+ * the error messages resulting from libdc's return codes
+ */
+const char *errmsg (dc_status_t rc)
+{
+	switch (rc) {
+	case DC_STATUS_SUCCESS:
+		return "Success";
+	case DC_STATUS_UNSUPPORTED:
+		return "Unsupported operation";
+	case DC_STATUS_INVALIDARGS:
+		return "Invalid arguments";
+	case DC_STATUS_NOMEMORY:
+		return "Out of memory";
+	case DC_STATUS_NODEVICE:
+		return "No device found";
+	case DC_STATUS_NOACCESS:
+		return "Access denied";
+	case DC_STATUS_IO:
+		return "Input/output error";
+	case DC_STATUS_TIMEOUT:
+		return "Timeout";
+	case DC_STATUS_PROTOCOL:
+		return "Protocol error";
+	case DC_STATUS_DATAFORMAT:
+		return "Data format error";
+	case DC_STATUS_CANCELLED:
+		return "Cancelled";
+	default:
+		return "Unknown error";
+	}
+}
+
 static dc_status_t create_parser(device_data_t *devdata, dc_parser_t **parser)
 {
 	return dc_parser_new(parser, devdata->device);
 }
 
-static int parse_gasmixes(device_data_t *devdata, struct dive *dive, dc_parser_t *parser, int ngases,
-			  const unsigned char *data)
+static int parse_gasmixes(device_data_t *devdata, struct dive *dive, dc_parser_t *parser, int ngases)
 {
 	static bool shown_warning = false;
 	int i, rc;
@@ -101,13 +137,19 @@ static int parse_gasmixes(device_data_t *devdata, struct dive *dive, dc_parser_t
 		}
 		if (!IS_FP_SAME(tank.volume, 0.0))
 			no_volume = false;
+
+		// this new API also gives us the beginning and end pressure for the tank
+		if (!IS_FP_SAME(tank.beginpressure, 0.0) && !IS_FP_SAME(tank.endpressure, 0.0)) {
+			dive->cylinder[i].start.mbar = tank.beginpressure * 1000;
+			dive->cylinder[i].end.mbar = tank.endpressure * 1000;
+		}
 #endif
 		if (no_volume) {
 			/* for the first tank, if there is no tanksize available from the
 			 * dive computer, fill in the default tank information (if set) */
 			fill_default_cylinder(&dive->cylinder[i]);
 		}
-		/* whatever happens, make sure there is a name for the cylidner */
+		/* whatever happens, make sure there is a name for the cylinder */
 		if (same_string(dive->cylinder[i].type.description, ""))
 			dive->cylinder[i].type.description = strdup(translate("gettextFromC", "unknown"));
 	}
@@ -157,6 +199,7 @@ void
 sample_cb(dc_sample_type_t type, dc_sample_value_t value, void *userdata)
 {
 	unsigned int mm;
+	static unsigned int nsensor = 0;
 	struct divecomputer *dc = userdata;
 	struct sample *sample;
 
@@ -175,6 +218,7 @@ sample_cb(dc_sample_type_t type, dc_sample_value_t value, void *userdata)
 
 	switch (type) {
 	case DC_SAMPLE_TIME:
+		nsensor = 0;
 		mm = 0;
 		if (sample) {
 			sample->in_deco = in_deco;
@@ -204,7 +248,7 @@ sample_cb(dc_sample_type_t type, dc_sample_value_t value, void *userdata)
 		handle_event(dc, sample, value);
 		break;
 	case DC_SAMPLE_RBT:
-		printf("   <rbt>%u</rbt>\n", value.rbt);
+		sample->rbt.seconds = (!strncasecmp(dc->model, "suunto", 6)) ? value.rbt : value.rbt * 60;
 		break;
 	case DC_SAMPLE_HEARTBEAT:
 		sample->heartbeat = value.heartbeat;
@@ -227,7 +271,14 @@ sample_cb(dc_sample_type_t type, dc_sample_value_t value, void *userdata)
 		sample->setpoint.mbar = po2 = rint(value.setpoint * 1000);
 		break;
 	case DC_SAMPLE_PPO2:
-		sample->setpoint.mbar = po2 = rint(value.ppo2 * 1000);
+		if (nsensor < 3)
+			sample->o2sensor[nsensor].mbar = rint(value.ppo2 * 1000);
+		else
+			report_error("%d is more o2 sensors than we can handle", nsensor);
+		nsensor++;
+		// Set the amount of detected o2 sensors
+		if (nsensor > dc->no_o2sensors)
+			dc->no_o2sensors = nsensor;
 		break;
 	case DC_SAMPLE_CNS:
 		sample->cns = cns = rint(value.cns * 100);
@@ -373,7 +424,7 @@ static uint32_t calculate_diveid(const unsigned char *fingerprint, unsigned int
 #ifdef DC_FIELD_STRING
 static uint32_t calculate_string_hash(const char *str)
 {
-	return calculate_diveid(str, strlen(str));
+	return calculate_diveid((const unsigned char *)str, strlen(str));
 }
 
 static void parse_string_field(struct dive *dive, dc_field_string_t *str)
@@ -400,45 +451,19 @@ static void parse_string_field(struct dive *dive, dc_field_string_t *str)
 }
 #endif
 
-/* returns true if we want libdivecomputer's dc_device_foreach() to continue,
- *  false otherwise */
-static int dive_cb(const unsigned char *data, unsigned int size,
-		   const unsigned char *fingerprint, unsigned int fsize,
-		   void *userdata)
+static dc_status_t libdc_header_parser(dc_parser_t *parser, struct device_data_t *devdata, struct dive *dive)
 {
-	int rc;
-	dc_parser_t *parser = NULL;
-	device_data_t *devdata = userdata;
+	dc_status_t rc = 0;
 	dc_datetime_t dt = { 0 };
 	struct tm tm;
-	struct dive *dive = NULL;
 
-	/* reset the deco / ndl data */
-	ndl = stoptime = stopdepth = 0;
-	in_deco = false;
-
-	rc = create_parser(devdata, &parser);
-	if (rc != DC_STATUS_SUCCESS) {
-		dev_info(devdata, translate("gettextFromC", "Unable to create parser for %s %s"), devdata->vendor, devdata->product);
-		return false;
-	}
-
-	rc = dc_parser_set_data(parser, data, size);
-	if (rc != DC_STATUS_SUCCESS) {
-		dev_info(devdata, translate("gettextFromC", "Error registering the data"));
-		goto error_exit;
-	}
-
-	import_dive_number++;
-	dive = alloc_dive();
 	rc = dc_parser_get_datetime(parser, &dt);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
 		dev_info(devdata, translate("gettextFromC", "Error parsing the datetime"));
-		goto error_exit;
+		return rc;
 	}
-	dive->dc.model = strdup(devdata->model);
+
 	dive->dc.deviceid = devdata->deviceid;
-	dive->dc.diveid = calculate_diveid(fingerprint, fsize);
 
 	if (rc == DC_STATUS_SUCCESS) {
 		tm.tm_year = dt.year;
@@ -449,13 +474,17 @@ static int dive_cb(const unsigned char *data, unsigned int size,
 		tm.tm_sec = dt.second;
 		dive->when = dive->dc.when = utc_mktime(&tm);
 	}
+
 	// Parse the divetime.
-	dev_info(devdata, translate("gettextFromC", "Dive %d: %s"), import_dive_number, get_dive_date_c_string(dive->when));
+	const char *date_string = get_dive_date_c_string(dive->when);
+	dev_info(devdata, translate("gettextFromC", "Dive %d: %s"), import_dive_number, date_string);
+	free((void *)date_string);
+
 	unsigned int divetime = 0;
 	rc = dc_parser_get_field(parser, DC_FIELD_DIVETIME, 0, &divetime);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
 		dev_info(devdata, translate("gettextFromC", "Error parsing the divetime"));
-		goto error_exit;
+		return rc;
 	}
 	if (rc == DC_STATUS_SUCCESS)
 		dive->dc.duration.seconds = divetime;
@@ -465,7 +494,7 @@ static int dive_cb(const unsigned char *data, unsigned int size,
 	rc = dc_parser_get_field(parser, DC_FIELD_MAXDEPTH, 0, &maxdepth);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
 		dev_info(devdata, translate("gettextFromC", "Error parsing the maxdepth"));
-		goto error_exit;
+		return rc;
 	}
 	if (rc == DC_STATUS_SUCCESS)
 		dive->dc.maxdepth.mm = rint(maxdepth * 1000);
@@ -484,7 +513,7 @@ static int dive_cb(const unsigned char *data, unsigned int size,
 		rc = dc_parser_get_field(parser, temp_fields[i], 0, &temperature);
 		if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
 			dev_info(devdata, translate("gettextFromC", "Error parsing temperature"));
-			goto error_exit;
+			return rc;
 		}
 		if (rc == DC_STATUS_SUCCESS)
 			switch(i) {
@@ -504,7 +533,7 @@ static int dive_cb(const unsigned char *data, unsigned int size,
 	rc = dc_parser_get_field(parser, DC_FIELD_GASMIX_COUNT, 0, &ngases);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
 		dev_info(devdata, translate("gettextFromC", "Error parsing the gas mix count"));
-		goto error_exit;
+		return rc;
 	}
 
 #if DC_VERSION_CHECK(0, 3, 0)
@@ -516,7 +545,7 @@ static int dive_cb(const unsigned char *data, unsigned int size,
 	rc = dc_parser_get_field(parser, DC_FIELD_SALINITY, 0, &salinity);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
 		dev_info(devdata, translate("gettextFromC", "Error obtaining water salinity"));
-		goto error_exit;
+		return rc;
 	}
 	if (rc == DC_STATUS_SUCCESS)
 		dive->dc.salinity = rint(salinity.density * 10.0);
@@ -525,7 +554,7 @@ static int dive_cb(const unsigned char *data, unsigned int size,
 	rc = dc_parser_get_field(parser, DC_FIELD_ATMOSPHERIC, 0, &surface_pressure);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
 		dev_info(devdata, translate("gettextFromC", "Error obtaining surface pressure"));
-		goto error_exit;
+		return rc;
 	}
 	if (rc == DC_STATUS_SUCCESS)
 		dive->dc.surface_pressure.mbar = rint(surface_pressure * 1000.0);
@@ -550,7 +579,7 @@ static int dive_cb(const unsigned char *data, unsigned int size,
 	rc = dc_parser_get_field(parser, DC_FIELD_DIVEMODE, 0, &divemode);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
 		dev_info(devdata, translate("gettextFromC", "Error obtaining divemode"));
-		goto error_exit;
+		return rc;
 	}
 	if (rc == DC_STATUS_SUCCESS)
 		switch(divemode) {
@@ -567,12 +596,55 @@ static int dive_cb(const unsigned char *data, unsigned int size,
 		}
 #endif
 
-	rc = parse_gasmixes(devdata, dive, parser, ngases, data);
-	if (rc != DC_STATUS_SUCCESS) {
+	rc = parse_gasmixes(devdata, dive, parser, ngases);
+	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
 		dev_info(devdata, translate("gettextFromC", "Error parsing the gas mix"));
+		return rc;
+	}
+
+	return DC_STATUS_SUCCESS;
+}
+
+/* returns true if we want libdivecomputer's dc_device_foreach() to continue,
+ *  false otherwise */
+static int dive_cb(const unsigned char *data, unsigned int size,
+		   const unsigned char *fingerprint, unsigned int fsize,
+		   void *userdata)
+{
+	int rc;
+	dc_parser_t *parser = NULL;
+	device_data_t *devdata = userdata;
+	struct dive *dive = NULL;
+
+	/* reset the deco / ndl data */
+	ndl = stoptime = stopdepth = 0;
+	in_deco = false;
+
+	rc = create_parser(devdata, &parser);
+	if (rc != DC_STATUS_SUCCESS) {
+		dev_info(devdata, translate("gettextFromC", "Unable to create parser for %s %s"), devdata->vendor, devdata->product);
+		return false;
+	}
+
+	rc = dc_parser_set_data(parser, data, size);
+	if (rc != DC_STATUS_SUCCESS) {
+		dev_info(devdata, translate("gettextFromC", "Error registering the data"));
 		goto error_exit;
 	}
 
+	import_dive_number++;
+	dive = alloc_dive();
+
+	// Parse the dive's header data
+	rc = libdc_header_parser (parser, devdata, dive);
+	if (rc != DC_STATUS_SUCCESS) {
+		dev_info(devdata, translate("getextFromC", "Error parsing the header"));
+		goto error_exit;
+	}
+
+	dive->dc.model = strdup(devdata->model);
+	dive->dc.diveid = calculate_diveid(fingerprint, fsize);
+
 	// Initialize the sample data.
 	rc = parse_samples(devdata, &dive->dc, parser);
 	if (rc != DC_STATUS_SUCCESS) {
@@ -818,7 +890,7 @@ static const char *do_device_import(device_data_t *data)
 	return NULL;
 }
 
-static void
+void
 logfunc(dc_context_t *context, dc_loglevel_t loglevel, const char *file, unsigned int line, const char *function, const char *msg, void *userdata)
 {
 	const char *loglevels[] = { "NONE", "ERROR", "WARNING", "INFO", "DEBUG", "ALL" };
@@ -858,14 +930,38 @@ const char *do_libdivecomputer_import(device_data_t *data)
 	}
 
 	err = translate("gettextFromC", "Unable to open %s %s (%s)");
-	rc = dc_device_open(&data->device, data->context, data->descriptor, data->devname);
+
+#if defined(SSRF_CUSTOM_SERIAL)
+	dc_serial_t *serial_device = NULL;
+
+	if (data->bluetooth_mode) {
+		rc = dc_serial_qt_open(&serial_device, data->context, data->devname);
+#ifdef SERIAL_FTDI
+	} else if (!strcmp(data->devname, "ftdi")) {
+		rc = dc_serial_ftdi_open(&serial_device, data->context);
+#endif
+	}
+
+	if (rc != DC_STATUS_SUCCESS) {
+		report_error(errmsg(rc));
+	} else if (serial_device) {
+		rc = dc_device_custom_open(&data->device, data->context, data->descriptor, serial_device);
+	} else {
+#else
+	{
+#endif
+		rc = dc_device_open(&data->device, data->context, data->descriptor, data->devname);
+
+		if (rc != DC_STATUS_SUCCESS && subsurface_access(data->devname, R_OK | W_OK) != 0)
+			err = translate("gettextFromC", "Insufficient privileges to open the device %s %s (%s)");
+	}
+
 	if (rc == DC_STATUS_SUCCESS) {
 		err = do_device_import(data);
 		/* TODO: Show the logfile to the user on error. */
 		dc_device_close(data->device);
 		data->device = NULL;
-	} else if (subsurface_access(data->devname, R_OK | W_OK) != 0)
-		err = translate("gettextFromC", "Insufficient privileges to open the device %s %s (%s)");
+	}
 
 	dc_context_free(data->context);
 	data->context = NULL;
@@ -876,3 +972,74 @@ const char *do_libdivecomputer_import(device_data_t *data)
 
 	return err;
 }
+
+/*
+ * Parse data buffers instead of dc devices downloaded data.
+ * Intended to be used to parse profile data from binary files during import tasks.
+ * Actually included Uwatec families because of works on datatrak and smartrak logs
+ * and OSTC families for OSTCTools logs import.
+ * For others, simply include them in the switch  (check parameters).
+ * Note that dc_descriptor_t in data  *must* have been filled using dc_descriptor_iterator()
+ * calls.
+ */
+dc_status_t libdc_buffer_parser(struct dive *dive, device_data_t *data, unsigned char *buffer, int size)
+{
+	dc_status_t rc;
+	dc_parser_t *parser = NULL;
+
+	switch (data->descriptor->type) {
+	case DC_FAMILY_UWATEC_ALADIN:
+	case DC_FAMILY_UWATEC_MEMOMOUSE:
+		rc = uwatec_memomouse_parser_create(&parser, data->context, 0, 0);
+		break;
+	case DC_FAMILY_UWATEC_SMART:
+	case DC_FAMILY_UWATEC_MERIDIAN:
+		rc = uwatec_smart_parser_create (&parser, data->context, data->descriptor->model, 0, 0);
+		break;
+	case DC_FAMILY_HW_OSTC:
+#if defined(SSRF_CUSTOM_SERIAL)
+		rc = hw_ostc_parser_create (&parser, data->context, data->deviceid, 0);
+#else
+		rc = hw_ostc_parser_create (&parser, data->context, data->deviceid);
+#endif
+		break;
+	case DC_FAMILY_HW_FROG:
+	case DC_FAMILY_HW_OSTC3:
+#if defined(SSRF_CUSTOM_SERIAL)
+		rc = hw_ostc_parser_create (&parser, data->context, data->deviceid, 1);
+#else
+		rc = hw_ostc_parser_create (&parser, data->context, data->deviceid);
+#endif
+		break;
+	default:
+		report_error("Device type not handled!");
+		return DC_STATUS_UNSUPPORTED;
+	}
+	if  (rc != DC_STATUS_SUCCESS) {
+		report_error("Error creating parser.");
+		dc_parser_destroy (parser);
+		return rc;
+	}
+	rc = dc_parser_set_data(parser, buffer, size);
+	if (rc != DC_STATUS_SUCCESS) {
+		report_error("Error registering the data.");
+		dc_parser_destroy (parser);
+		return rc;
+	}
+	// Do not parse Aladin/Memomouse headers as they are fakes
+	// Do not return on error, we can still parse the samples
+	if (data->descriptor->type != DC_FAMILY_UWATEC_ALADIN && data->descriptor->type != DC_FAMILY_UWATEC_MEMOMOUSE) {
+		rc = libdc_header_parser (parser, data, dive);
+		if (rc != DC_STATUS_SUCCESS) {
+			report_error("Error parsing the dive header data. Dive # %d\nStatus = %s", dive->number, errmsg(rc));
+		}
+	}
+	rc = dc_parser_samples_foreach (parser, sample_cb, &dive->dc);
+	if (rc != DC_STATUS_SUCCESS) {
+		report_error("Error parsing the sample data. Dive # %d\nStatus = %s", dive->number, errmsg(rc));
+		dc_parser_destroy (parser);
+		return rc;
+	}
+	dc_parser_destroy(parser);
+	return(DC_STATUS_SUCCESS);
+}
