@@ -10,12 +10,25 @@
 #include <fcntl.h>
 #include <git2.h>
 
+#include "gettext.h"
+
 #include "dive.h"
+#include "divelist.h"
 #include "device.h"
 #include "membuffer.h"
+#include "git-access.h"
+#include "qthelperfromc.h"
 
 const char *saved_git_id = NULL;
 
+struct picture_entry_list {
+	void *data;
+	int len;
+	const char *hash;
+	struct picture_entry_list *next;
+};
+struct picture_entry_list *pel = NULL;
+
 struct keyword_action {
 	const char *keyword;
 	void (*fn)(char *, struct membuffer *, void *);
@@ -23,12 +36,20 @@ struct keyword_action {
 #define ARRAY_SIZE(array) (sizeof(array)/sizeof(array[0]))
 
 extern degrees_t parse_degrees(char *buf, char **end);
-static void parse_dive_gps(char *line, struct membuffer *str, void *_dive)
-{
-	struct dive *dive = _dive;
+git_blob *git_tree_entry_blob(git_repository *repo, const git_tree_entry *entry);
 
-	dive->latitude = parse_degrees(line, &line);
-	dive->longitude = parse_degrees(line, &line);
+static void save_picture_from_git(struct picture *picture)
+{
+	struct picture_entry_list *pic_entry = pel;
+
+	while (pic_entry) {
+		if (same_string(pic_entry->hash, picture->hash)) {
+			savePictureLocal(picture, pic_entry->data, pic_entry->len);
+			return;
+		}
+		pic_entry = pic_entry->next;
+	}
+	fprintf(stderr, "didn't find picture entry for %s\n", picture->filename);
 }
 
 static char *get_utf8(struct membuffer *b)
@@ -145,8 +166,58 @@ static int get_index(const char *line)
 static int get_hex(const char *line)
 { return strtoul(line, NULL, 16); }
 
+/* this is in qthelper.cpp, so including the .h file is a pain */
+extern const char *printGPSCoords(int lat, int lon);
+
+static void parse_dive_gps(char *line, struct membuffer *str, void *_dive)
+{
+	uint32_t uuid;
+	degrees_t latitude = parse_degrees(line, &line);
+	degrees_t longitude = parse_degrees(line, &line);
+	struct dive *dive = _dive;
+	struct dive_site *ds = get_dive_site_for_dive(dive);
+	if (!ds) {
+		uuid = get_dive_site_uuid_by_gps(latitude, longitude, NULL);
+		if (!uuid)
+			uuid = create_dive_site_with_gps("", latitude, longitude, dive->when);
+		dive->dive_site_uuid = uuid;
+	} else {
+		if (dive_site_has_gps_location(ds) &&
+		    (ds->latitude.udeg != latitude.udeg || ds->longitude.udeg != longitude.udeg)) {
+			const char *coords = printGPSCoords(latitude.udeg, longitude.udeg);
+			// we have a dive site that already has GPS coordinates
+			ds->notes = add_to_string(ds->notes, translate("gettextFromC", "multiple GPS locations for this dive site; also %s\n"), coords);
+			free((void *)coords);
+		}
+		ds->latitude = latitude;
+		ds->longitude = longitude;
+	}
+
+}
+
 static void parse_dive_location(char *line, struct membuffer *str, void *_dive)
-{ struct dive *dive = _dive; dive->location = get_utf8(str); }
+{
+	uint32_t uuid;
+	char *name = get_utf8(str);
+	struct dive *dive = _dive;
+	struct dive_site *ds = get_dive_site_for_dive(dive);
+	if (!ds) {
+		uuid = get_dive_site_uuid_by_name(name, NULL);
+		if (!uuid)
+			uuid = create_dive_site(name, dive->when);
+		dive->dive_site_uuid = uuid;
+	} else {
+		// we already had a dive site linked to the dive
+		if (same_string(ds->name, "")) {
+			ds->name = strdup(name);
+		} else {
+			// and that dive site had a name. that's weird - if our name is different, add it to the notes
+			if (!same_string(ds->name, name))
+				ds->notes = add_to_string(ds->notes, translate("gettextFromC", "additional name for site: %s\n"), name);
+		}
+	}
+	free(name);
+}
 
 static void parse_dive_divemaster(char *line, struct membuffer *str, void *_dive)
 { struct dive *dive = _dive; dive->divemaster = get_utf8(str); }
@@ -160,6 +231,9 @@ static void parse_dive_suit(char *line, struct membuffer *str, void *_dive)
 static void parse_dive_notes(char *line, struct membuffer *str, void *_dive)
 { struct dive *dive = _dive; dive->notes = get_utf8(str); }
 
+static void parse_dive_divesiteid(char *line, struct membuffer *str, void *_dive)
+{ struct dive *dive = _dive; dive->dive_site_uuid = get_hex(line); }
+
 /*
  * We can have multiple tags in the membuffer. They are separated by
  * NUL bytes.
@@ -205,6 +279,38 @@ static void parse_dive_visibility(char *line, struct membuffer *str, void *_dive
 static void parse_dive_notrip(char *line, struct membuffer *str, void *_dive)
 { struct dive *dive = _dive; dive->tripflag = NO_TRIP; }
 
+static void parse_site_description(char *line, struct membuffer *str, void *_ds)
+{ struct dive_site *ds = _ds; ds->description = strdup(mb_cstring(str)); }
+
+static void parse_site_name(char *line, struct membuffer *str, void *_ds)
+{ struct dive_site *ds = _ds; ds->name = strdup(mb_cstring(str)); }
+
+static void parse_site_notes(char *line, struct membuffer *str, void *_ds)
+{ struct dive_site *ds = _ds; ds->notes = strdup(mb_cstring(str)); }
+
+extern degrees_t parse_degrees(char *buf, char **end);
+static void parse_site_gps(char *line, struct membuffer *str, void *_ds)
+{
+	struct dive_site *ds = _ds;
+
+	ds->latitude = parse_degrees(line, &line);
+	ds->longitude = parse_degrees(line, &line);
+}
+
+static void parse_site_geo(char *line, struct membuffer *str, void *_ds)
+{
+	struct dive_site *ds = _ds;
+	if (ds->taxonomy.category == NULL)
+		ds->taxonomy.category = alloc_taxonomy();
+	int nr = ds->taxonomy.nr;
+	if (nr < TC_NR_CATEGORIES) {
+		struct taxonomy *t = &ds->taxonomy.category[nr];
+		t->value = strdup(mb_cstring(str));
+		sscanf(line, "cat %d origin %d \"", &t->category, (int *)&t->origin);
+		ds->taxonomy.nr++;
+	}
+}
+
 /* Parse key=val parts of samples and cylinders etc */
 static char *parse_keyvalue_entry(void (*fn)(void *, const char *, const char *), void *fndata, char *line)
 {
@@ -391,6 +497,12 @@ static void parse_sample_keyvalue(void *_sample, const char *key, const char *va
 		sample->cns = atoi(value);
 		return;
 	}
+
+	if (!strcmp(key, "rbt")) {
+		sample->rbt = get_duration(value);
+		return;
+	}
+
 	if (!strcmp(key, "po2")) {
 		pressure_t p = get_pressure(value);
 		sample->setpoint.mbar = p.mbar;
@@ -659,6 +771,12 @@ static void parse_trip_notes(char *line, struct membuffer *str, void *_trip)
 static void parse_settings_autogroup(char *line, struct membuffer *str, void *_unused)
 { set_autogroup(1); }
 
+static void parse_settings_units(char *line, struct membuffer *str, void *unused)
+{
+	if (line)
+		set_informational_units(line);
+}
+
 static void parse_settings_userid(char *line, struct membuffer *str, void *_unused)
 {
 	if (line) {
@@ -671,13 +789,15 @@ static void parse_settings_userid(char *line, struct membuffer *str, void *_unus
  * Our versioning is a joke right now, but this is more of an example of what we
  * *can* do some day. And if we do change the version, this warning will show if
  * you read with a version of subsurface that doesn't know about it.
+ * We MUST keep this in sync with the XML version (so we can report a consistent
+ * minimum datafile version)
  */
-#define VERSION 2
 static void parse_settings_version(char *line, struct membuffer *str, void *_unused)
 {
 	int version = atoi(line);
-	if (version > VERSION)
-		report_error("Git save file version %d is newer than version %d I know about", version, VERSION);
+	report_datafile_version(version);
+	if (version > DATAFORMAT_VERSION)
+		report_error("Git save file version %d is newer than version %d I know about", version, DATAFORMAT_VERSION);
 }
 
 /* The string in the membuffer is the version string of subsurface that saved things, just FYI */
@@ -761,6 +881,12 @@ static void parse_picture_gps(char *line, struct membuffer *str, void *_pic)
 	pic->longitude = parse_degrees(line, &line);
 }
 
+static void parse_picture_hash(char *line, struct membuffer *str, void *_pic)
+{
+	struct picture *pic = _pic;
+	pic->hash = get_utf8(str);
+}
+
 /* These need to be sorted! */
 struct keyword_action dc_action[] = {
 #undef D
@@ -783,7 +909,7 @@ static void divecomputer_parser(char *line, struct membuffer *str, void *_dc)
 struct keyword_action dive_action[] = {
 #undef D
 #define D(x) { #x, parse_dive_ ## x }
-	D(airtemp), D(buddy), D(cylinder), D(divemaster), D(duration),
+	D(airtemp), D(buddy), D(cylinder), D(divemaster), D(divesiteid), D(duration),
 	D(gps), D(location), D(notes), D(notrip), D(rating), D(suit),
 	D(tags), D(visibility), D(watertemp), D(weightsystem)
 };
@@ -793,6 +919,18 @@ static void dive_parser(char *line, struct membuffer *str, void *_dive)
 	match_action(line, str, _dive, dive_action, ARRAY_SIZE(dive_action));
 }
 
+/* These need to be sorted! */
+struct keyword_action site_action[] = {
+#undef D
+#define D(x) { #x, parse_site_ ## x }
+	D(description), D(geo), D(gps), D(name), D(notes)
+};
+
+static void site_parser(char *line, struct membuffer *str, void *_ds)
+{
+	match_action(line, str, _ds, site_action, ARRAY_SIZE(site_action));
+}
+
 /* These need to be sorted! */
 struct keyword_action trip_action[] = {
 #undef D
@@ -809,7 +947,7 @@ static void trip_parser(char *line, struct membuffer *str, void *_trip)
 static struct keyword_action settings_action[] = {
 #undef D
 #define D(x) { #x, parse_settings_ ## x }
-	D(autogroup), D(divecomputerid), D(subsurface), D(userid), D(version),
+	D(autogroup), D(divecomputerid), D(subsurface), D(units), D(userid), D(version),
 };
 
 static void settings_parser(char *line, struct membuffer *str, void *_unused)
@@ -821,7 +959,7 @@ static void settings_parser(char *line, struct membuffer *str, void *_unused)
 static struct keyword_action picture_action[] = {
 #undef D
 #define D(x) { #x, parse_picture_ ## x }
-	D(filename), D(gps)
+	D(filename), D(gps), D(hash)
 };
 
 static void picture_parser(char *line, struct membuffer *str, void *_pic)
@@ -974,6 +1112,18 @@ static void finish_active_dive(void)
 	struct dive *dive = active_dive;
 
 	if (dive) {
+		/* check if we need to save pictures */
+		FOR_EACH_PICTURE(dive) {
+			if (!picture_exists(picture))
+				save_picture_from_git(picture);
+		}
+		/* free any memory we allocated to track pictures */
+		while (pel) {
+			free(pel->data);
+			void *lastone = pel;
+			pel = pel->next;
+			free(lastone);
+		}
 		active_dive = NULL;
 		record_dive(dive);
 	}
@@ -1037,7 +1187,8 @@ static int dive_trip_directory(const char *root, const char *name)
 }
 
 /*
- * Dive directory, name is [[yyyy-]mm-]nn-ddd-hh:mm:ss[~hex],
+ * Dive directory, name is [[yyyy-]mm-]nn-ddd-hh:mm:ss[~hex] in older git repositories
+ * but [[yyyy-]mm-]nn-ddd-hh=mm=ss[~hex] in newer repos as ':' is an illegal character for Windows files
  * and 'timeoff' points to what should be the time part of
  * the name (the first digit of the hour).
  *
@@ -1064,8 +1215,8 @@ static int dive_directory(const char *root, const char *name, int timeoff)
 	if (mday_off < 0)
 		return GIT_WALK_SKIP;
 
-	/* Get the time of day */
-	if (sscanf(name+timeoff, "%d:%d:%d", &h, &m, &s) != 3)
+	/* Get the time of day -- parse both time formats so we can read old repos when not on Windows */
+	if (sscanf(name+timeoff, "%d:%d:%d", &h, &m, &s) != 3 && sscanf(name+timeoff, "%d=%d=%d", &h, &m, &s) != 3)
 		return GIT_WALK_SKIP;
 	if (!validate_time(h, m, s))
 		return GIT_WALK_SKIP;
@@ -1166,7 +1317,10 @@ static int nonunique_length(const char *str)
  *
  *  - It's a dive directory. The name will be of the form
  *
- *       [[yyyy-]mm-]nn-ddd-hh:mm:ss[~hex]
+ *       [[yyyy-]mm-]nn-ddd-hh=mm=ss[~hex]
+ *
+ *    (older versions had this as [[yyyy-]mm-]nn-ddd-hh:mm:ss[~hex]
+ *     but that faile on Windows)
  *
  *    which describes the date and time of a dive (yyyy and mm
  *    are optional, and may be encoded in the path leading up to
@@ -1176,10 +1330,8 @@ static int nonunique_length(const char *str)
  *
  *  - It's some random non-dive-data directory.
  *
- *    Subsurface doesn't create these yet, but maybe we'll encode
- *    pictures etc. If it doesn't match the above patterns, we'll
- *    ignore them for dive loading purposes, and not even recurse
- *    into them.
+ *    If it doesn't match the above patterns, we'll ignore them
+ *    for dive loading purposes, and not even recurse into them.
  */
 static int walk_tree_directory(const char *root, const git_tree_entry *entry)
 {
@@ -1190,6 +1342,9 @@ static int walk_tree_directory(const char *root, const git_tree_entry *entry)
 	if (!strcmp(name, "Pictures"))
 		return picture_directory(root, name);
 
+	if (!strcmp(name, "01-Divesites"))
+		return GIT_WALK_OK;
+
 	while (isdigit(c = name[digits]))
 		digits++;
 
@@ -1212,7 +1367,7 @@ static int walk_tree_directory(const char *root, const git_tree_entry *entry)
 	 * We know the len is at least 3, because we had at least
 	 * two digits and a dash
 	 */
-	if (name[len-3] == ':')
+	if (name[len-3] == ':' || name[len-3] == '=')
 		return dive_directory(root, name, len-8);
 
 	if (digits != 2)
@@ -1284,6 +1439,20 @@ static int parse_dive_entry(git_repository *repo, const git_tree_entry *entry, c
 	return 0;
 }
 
+static int parse_site_entry(git_repository *repo, const git_tree_entry *entry, const char *suffix)
+{
+	if (*suffix == '\0')
+		return report_error("Dive site without uuid");
+	uint32_t uuid = strtoul(suffix, NULL, 16);
+	struct dive_site *ds = alloc_or_get_dive_site(uuid);
+	git_blob *blob = git_tree_entry_blob(repo, entry);
+	if (!blob)
+		return report_error("Unable to read dive site file");
+	for_each_line(blob, site_parser, ds);
+	git_blob_free(blob);
+	return 0;
+}
+
 static int parse_trip_entry(git_repository *repo, const git_tree_entry *entry)
 {
 	git_blob *blob = git_tree_entry_blob(repo, entry);
@@ -1300,12 +1469,29 @@ static int parse_settings_entry(git_repository *repo, const git_tree_entry *entr
 	if (!blob)
 		return report_error("Unable to read settings file");
 	set_save_userid_local(false);
-	set_userid("");
 	for_each_line(blob, settings_parser, NULL);
 	git_blob_free(blob);
 	return 0;
 }
 
+static int parse_picture_file(git_repository *repo, const git_tree_entry *entry, const char *name)
+{
+	/* remember the picture data so we can handle it when all dive data has been loaded
+	 * the name of the git file is PIC-<hash> */
+	git_blob *blob = git_tree_entry_blob(repo, entry);
+	const void *rawdata = git_blob_rawcontent(blob);
+	int len = git_blob_rawsize(blob);
+	struct picture_entry_list *new_pel = malloc(sizeof(struct picture_entry_list));
+	new_pel->next = pel;
+	pel = new_pel;
+	pel->data = malloc(len);
+	memcpy(pel->data, rawdata, len);
+	pel->len = len;
+	pel->hash = strdup(name + 4);
+	git_blob_free(blob);
+	return 0;
+}
+
 static int parse_picture_entry(git_repository *repo, const git_tree_entry *entry, const char *name)
 {
 	git_blob *blob;
@@ -1314,12 +1500,13 @@ static int parse_picture_entry(git_repository *repo, const git_tree_entry *entry
 	char sign;
 
 	/*
-	 * The format of the picture name files is just the offset
-	 * within the dive in form [[+-]hh:mm:ss, possibly followed
-	 * by a hash to make the filename unique (which we can just
-	 * ignore).
+	 * The format of the picture name files is just the offset within
+	 * the dive in form [[+-]hh=mm=ss (previously [[+-]hh:mm:ss, but
+	 * that didn't work on Windows), possibly followed by a hash to
+	 * make the filename unique (which we can just ignore).
 	 */
-	if (sscanf(name, "%c%d:%d:%d", &sign, &hh, &mm, &ss) != 4)
+	if (sscanf(name, "%c%d:%d:%d", &sign, &hh, &mm, &ss) != 4 &&
+	    sscanf(name, "%c%d=%d=%d", &sign, &hh, &mm, &ss) != 4)
 		return report_error("Unknown file name %s", name);
 	offset = ss + 60*(mm + 60*hh);
 	if (sign == '-')
@@ -1327,7 +1514,7 @@ static int parse_picture_entry(git_repository *repo, const git_tree_entry *entry
 
 	blob = git_tree_entry_blob(repo, entry);
 	if (!blob)
-		return report_error("Unable to read trip file");
+		return report_error("Unable to read picture file");
 
 	pic = alloc_picture();
 	pic->offset.seconds = offset;
@@ -1343,7 +1530,8 @@ static int walk_tree_file(const char *root, const git_tree_entry *entry, git_rep
 	struct dive *dive = active_dive;
 	dive_trip_t *trip = active_trip;
 	const char *name = git_tree_entry_name(entry);
-
+	if (verbose > 1)
+		fprintf(stderr, "git load handling file %s\n", name);
 	switch (*name) {
 	/* Picture file? They are saved as time offsets in the dive */
 	case '-': case '+':
@@ -1356,12 +1544,20 @@ static int walk_tree_file(const char *root, const git_tree_entry *entry, git_rep
 		if (dive && !strncmp(name, "Dive", 4))
 			return parse_dive_entry(repo, entry, name+4);
 		break;
+	case 'S':
+		if (!strncmp(name, "Site", 4))
+			return parse_site_entry(repo, entry, name + 5);
+		break;
 	case '0':
 		if (trip && !strcmp(name, "00-Trip"))
 			return parse_trip_entry(repo, entry);
 		if (!strcmp(name, "00-Subsurface"))
 			return parse_settings_entry(repo, entry);
 		break;
+	case 'P':
+		if (dive && !strncmp(name, "PIC-", 4))
+			return parse_picture_file(repo, entry, name);
+		break;
 	}
 	report_error("Unknown file %s%s (%p %p)", root, name, dive, trip);
 	return GIT_WALK_SKIP;
