@@ -19,6 +19,9 @@
 
 //#define DEBUG_GAS 1
 
+#define MAX_PROFILE_DECO 7200
+
+
 int selected_dive = -1; /* careful: 0 is a valid value */
 unsigned int dc_number = 0;
 
@@ -336,7 +339,7 @@ int get_cylinder_index(struct dive *dive, struct event *ev)
 	return best;
 }
 
-struct event *get_next_event(struct event *event, char *name)
+struct event *get_next_event(struct event *event, const char *name)
 {
 	if (!name || !*name)
 		return NULL;
@@ -551,9 +554,11 @@ struct plot_data *populate_plot_entries(struct dive *dive, struct divecomputer *
 	 * but samples could be more dense than that (so add in dc->samples). We also
 	 * need to have one for every event (so count events and add that) and
 	 * additionally we want two surface events around the whole thing (thus the
-	 * additional 4).
+	 * additional 4).  There is also one extra space for a final entry
+	 * that has time > maxtime (because there can be surface samples
+	 * past "maxtime" in the original sample data)
 	 */
-	nr = dc->samples + 5 + maxtime / 10 + count_events(dc);
+	nr = dc->samples + 6 + maxtime / 10 + count_events(dc);
 	plot_data = calloc(nr, sizeof(struct plot_data));
 	pi->entry = plot_data;
 	if (!plot_data)
@@ -576,9 +581,9 @@ struct plot_data *populate_plot_entries(struct dive *dive, struct divecomputer *
 
 		/* Add intermediate plot entries if required */
 		delta = time - lasttime;
-		if (delta < 0) {
+		if (delta <= 0) {
 			time = lasttime;
-			delta = 0;
+			delta = 1; // avoid divide by 0
 		}
 		for (offset = 10; offset < delta; offset += 10) {
 			if (lasttime + offset > maxtime)
@@ -604,8 +609,6 @@ struct plot_data *populate_plot_entries(struct dive *dive, struct divecomputer *
 			ev = ev->next;
 		}
 
-		if (time > maxtime)
-			break;
 
 		entry->sec = time;
 		entry->depth = depth;
@@ -637,13 +640,17 @@ struct plot_data *populate_plot_entries(struct dive *dive, struct divecomputer *
 		entry->heartbeat = sample->heartbeat;
 		entry->bearing = sample->bearing.degrees;
 		entry->sac = sample->sac.mliter;
-
+		if (sample->rbt.seconds)
+			entry->rbt = sample->rbt.seconds;
 		/* skip events that happened at this time */
 		while (ev && ev->time.seconds == time)
 			ev = ev->next;
 		lasttime = time;
 		lastdepth = depth;
 		idx++;
+
+		if (time > maxtime)
+			break;
 	}
 
 	/* Add two final surface events */
@@ -757,7 +764,7 @@ static void setup_gas_sensor_pressure(struct dive *dive, struct divecomputer *dc
 }
 
 /* calculate DECO STOP / TTS / NDL */
-static void calculate_ndl_tts(double tissue_tolerance, struct plot_data *entry, struct dive *dive, double surface_pressure)
+static void calculate_ndl_tts(struct plot_data *entry, struct dive *dive, double surface_pressure)
 {
 	/* FIXME: This should be configurable */
 	/* ascent speed up to first deco stop */
@@ -770,23 +777,23 @@ static void calculate_ndl_tts(double tissue_tolerance, struct plot_data *entry,
 	const int time_stepsize = 60;
 	const int deco_stepsize = 3000;
 	/* at what depth is the current deco-step? */
-	int next_stop = ROUND_UP(deco_allowed_depth(tissue_tolerance, surface_pressure, dive, 1), deco_stepsize);
+	int next_stop = ROUND_UP(deco_allowed_depth(tissue_tolerance_calc(dive, depth_to_bar(entry->depth, dive)),
+						    surface_pressure, dive, 1), deco_stepsize);
 	int ascent_depth = entry->depth;
 	/* at what time should we give up and say that we got enuff NDL? */
-	const int max_ndl = 7200;
 	int cylinderindex = entry->cylinderindex;
 
 	/* If we don't have a ceiling yet, calculate ndl. Don't try to calculate
 	 * a ndl for lower values than 3m it would take forever */
 	if (next_stop == 0) {
 		if (entry->depth < 3000) {
-			entry->ndl = max_ndl;
+			entry->ndl = MAX_PROFILE_DECO;
 			return;
 		}
 		/* stop if the ndl is above max_ndl seconds, and call it plenty of time */
-		while (entry->ndl_calc < max_ndl && deco_allowed_depth(tissue_tolerance, surface_pressure, dive, 1) <= 0) {
+		while (entry->ndl_calc < MAX_PROFILE_DECO && deco_allowed_depth(tissue_tolerance_calc(dive, depth_to_bar(entry->depth, dive)), surface_pressure, dive, 1) <= 0) {
 			entry->ndl_calc += time_stepsize;
-			tissue_tolerance = add_segment(depth_to_mbar(entry->depth, dive) / 1000.0,
+			add_segment(depth_to_bar(entry->depth, dive),
 						       &dive->cylinder[cylinderindex].gasmix, time_stepsize, entry->o2pressure.mbar, dive, prefs.bottomsac);
 		}
 		/* we don't need to calculate anything else */
@@ -798,9 +805,9 @@ static void calculate_ndl_tts(double tissue_tolerance, struct plot_data *entry,
 
 	/* Add segments for movement to stopdepth */
 	for (; ascent_depth > next_stop; ascent_depth -= ascent_mm_per_step, entry->tts_calc += ascent_s_per_step) {
-		tissue_tolerance = add_segment(depth_to_mbar(ascent_depth, dive) / 1000.0,
-					       &dive->cylinder[cylinderindex].gasmix, ascent_s_per_step, entry->o2pressure.mbar, dive, prefs.decosac);
-		next_stop = ROUND_UP(deco_allowed_depth(tissue_tolerance, surface_pressure, dive, 1), deco_stepsize);
+		add_segment(depth_to_bar(ascent_depth, dive),
+			    &dive->cylinder[cylinderindex].gasmix, ascent_s_per_step, entry->o2pressure.mbar, dive, prefs.decosac);
+		next_stop = ROUND_UP(deco_allowed_depth(tissue_tolerance_calc(dive, depth_to_bar(ascent_depth, dive)), surface_pressure, dive, 1), deco_stepsize);
 	}
 	ascent_depth = next_stop;
 
@@ -816,13 +823,15 @@ static void calculate_ndl_tts(double tissue_tolerance, struct plot_data *entry,
 			entry->stoptime_calc += time_stepsize;
 
 		entry->tts_calc += time_stepsize;
-		tissue_tolerance = add_segment(depth_to_mbar(ascent_depth, dive) / 1000.0,
-					       &dive->cylinder[cylinderindex].gasmix, time_stepsize, entry->o2pressure.mbar, dive, prefs.decosac);
+		if (entry->tts_calc > MAX_PROFILE_DECO)
+			break;
+		add_segment(depth_to_bar(ascent_depth, dive),
+			    &dive->cylinder[cylinderindex].gasmix, time_stepsize, entry->o2pressure.mbar, dive, prefs.decosac);
 
-		if (deco_allowed_depth(tissue_tolerance, surface_pressure, dive, 1) <= next_stop) {
+		if (deco_allowed_depth(tissue_tolerance_calc(dive, depth_to_bar(ascent_depth,dive)), surface_pressure, dive, 1) <= next_stop) {
 			/* move to the next stop and add the travel between stops */
 			for (; ascent_depth > next_stop; ascent_depth -= ascent_mm_per_deco_step, entry->tts_calc += ascent_s_per_deco_step)
-				add_segment(depth_to_mbar(ascent_depth, dive) / 1000.0,
+				add_segment(depth_to_bar(ascent_depth, dive),
 					    &dive->cylinder[cylinderindex].gasmix, ascent_s_per_deco_step, entry->o2pressure.mbar, dive, prefs.decosac);
 			ascent_depth = next_stop;
 			next_stop -= deco_stepsize;
@@ -836,32 +845,36 @@ void calculate_deco_information(struct dive *dive, struct divecomputer *dc, stru
 {
 	int i;
 	double surface_pressure = (dc->surface_pressure.mbar ? dc->surface_pressure.mbar : get_surface_pressure_in_mbar(dive, true)) / 1000.0;
-	double tissue_tolerance = 0;
 	int last_ndl_tts_calc_time = 0;
 	for (i = 1; i < pi->nr; i++) {
 		struct plot_data *entry = pi->entry + i;
 		int j, t0 = (entry - 1)->sec, t1 = entry->sec;
 		int time_stepsize = 20;
 
-		entry->ambpressure = (double)depth_to_mbar(entry->depth, dive) / 1000.0;
+		entry->ambpressure = depth_to_bar(entry->depth, dive);
 		entry->gfline = MAX((double)prefs.gflow, (entry->ambpressure - surface_pressure) / (gf_low_pressure_this_dive - surface_pressure) *
 									 (prefs.gflow - prefs.gfhigh) +
 								 prefs.gfhigh) *
 					(100.0 - AMB_PERCENTAGE) / 100.0 + AMB_PERCENTAGE;
+		if (t0 > t1) {
+			fprintf(stderr, "non-monotonous dive stamps %d %d\n", t0, t1);
+			int xchg = t1;
+			t1 = t0;
+			t0 = xchg;
+		}
 		if (t0 != t1 && t1 - t0 < time_stepsize)
 			time_stepsize = t1 - t0;
 		for (j = t0 + time_stepsize; j <= t1; j += time_stepsize) {
 			int depth = interpolate(entry[-1].depth, entry[0].depth, j - t0, t1 - t0);
-			double min_pressure = add_segment(depth_to_mbar(depth, dive) / 1000.0,
-							  &dive->cylinder[entry->cylinderindex].gasmix, time_stepsize, entry->o2pressure.mbar, dive, entry->sac);
-			tissue_tolerance = min_pressure;
-			if (j - t0 < time_stepsize)
-				time_stepsize = j - t0;
+			add_segment(depth_to_bar(depth, dive),
+				    &dive->cylinder[entry->cylinderindex].gasmix, time_stepsize, entry->o2pressure.mbar, dive, entry->sac);
+			if ((t1 - j < time_stepsize) && (j < t1))
+				time_stepsize = t1 - j;
 		}
 		if (t0 == t1)
 			entry->ceiling = (entry - 1)->ceiling;
 		else
-			entry->ceiling = deco_allowed_depth(tissue_tolerance, surface_pressure, dive, !prefs.calcceiling3m);
+			entry->ceiling = deco_allowed_depth(tissue_tolerance_calc(dive, depth_to_bar(entry->depth, dive)), surface_pressure, dive, !prefs.calcceiling3m);
 		for (j = 0; j < 16; j++) {
 			double m_value = buehlmann_inertgas_a[j] + entry->ambpressure / buehlmann_inertgas_b[j];
 			entry->ceilings[j] = deco_allowed_depth(tolerated_by_tissue[j], surface_pressure, dive, 1);
@@ -886,10 +899,10 @@ void calculate_deco_information(struct dive *dive, struct divecomputer *dc, stru
 
 			/* We are going to mess up deco state, so store it for later restore */
 			char *cache_data = NULL;
-			cache_deco_state(tissue_tolerance, &cache_data);
-			calculate_ndl_tts(tissue_tolerance, entry, dive, surface_pressure);
+			cache_deco_state(&cache_data);
+			calculate_ndl_tts(entry, dive, surface_pressure);
 			/* Restore "real" deco state for next real time step */
-			tissue_tolerance = restore_deco_state(cache_data);
+			restore_deco_state(cache_data);
 			free(cache_data);
 		}
 	}
@@ -952,7 +965,7 @@ static void calculate_gas_information_new(struct dive *dive, struct plot_info *p
 		struct plot_data *entry = pi->entry + i;
 		int cylinderindex = entry->cylinderindex;
 
-		amb_pressure = depth_to_mbar(entry->depth, dive) / 1000.0;
+		amb_pressure = depth_to_bar(entry->depth, dive);
 
 		fill_pressures(&entry->pressures, amb_pressure, &dive->cylinder[cylinderindex].gasmix, entry->o2pressure.mbar / 1000.0, dive->dc.divemode);
 		fn2 = (int)(1000.0 * entry->pressures.n2 / amb_pressure);
@@ -963,7 +976,7 @@ static void calculate_gas_information_new(struct dive *dive, struct plot_info *p
 		 * END takes O₂ + N₂ (air) into account ("Narcotic" for trimix dives)
 		 * EAD just uses N₂ ("Air" for nitrox dives) */
 		pressure_t modpO2 = { .mbar = (int)(prefs.modpO2 * 1000) };
-		entry->mod = (double)gas_mod(&dive->cylinder[cylinderindex].gasmix, modpO2, 1).mm;
+		entry->mod = (double)gas_mod(&dive->cylinder[cylinderindex].gasmix, modpO2, dive, 1).mm;
 		entry->end = (entry->depth + 10000) * (1000 - fhe) / 1000.0 - 10000;
 		entry->ead = (entry->depth + 10000) * fn2 / (double)N2_IN_AIR - 10000;
 		entry->eadd = (entry->depth + 10000) *
@@ -1193,10 +1206,19 @@ static void plot_string(struct plot_info *pi, struct plot_data *entry, struct me
 		 * everything will be ok. */
 		put_string(b, translate("gettextFromC", "In deco (calc)\n"));
 	} else if (prefs.calcndltts && entry->ndl_calc != 0) {
-		put_format(b, translate("gettextFromC", "NDL: %umin (calc)\n"), DIV_UP(entry->ndl_calc, 60));
+		if(entry->ndl_calc < MAX_PROFILE_DECO)
+			put_format(b, translate("gettextFromC", "NDL: %umin (calc)\n"), DIV_UP(entry->ndl_calc, 60));
+		else
+			put_format(b, "%s", translate("gettextFromC", "NDL: >2h (calc)\n"));
 	}
-	if (entry->tts_calc)
-		put_format(b, translate("gettextFromC", "TTS: %umin (calc)\n"), DIV_UP(entry->tts_calc, 60));
+	if (entry->tts_calc) {
+		if (entry->tts_calc < MAX_PROFILE_DECO)
+			put_format(b, translate("gettextFromC", "TTS: %umin (calc)\n"), DIV_UP(entry->tts_calc, 60));
+		else
+			put_format(b, "%s", translate("gettextFromC", "TTS: >2h (calc)\n"));
+	}
+	if (entry->rbt)
+		put_format(b, translate("gettextFromC", "RBT: %umin\n"), DIV_UP(entry->rbt, 60));
 	if (entry->ceiling) {
 		depthvalue = get_depth_units(entry->ceiling, NULL, &depth_unit);
 		put_format(b, translate("gettextFromC", "Calculated ceiling %.0f%s\n"), depthvalue, depth_unit);
@@ -1205,7 +1227,7 @@ static void plot_string(struct plot_info *pi, struct plot_data *entry, struct me
 			for (k = 0; k < 16; k++) {
 				if (entry->ceilings[k]) {
 					depthvalue = get_depth_units(entry->ceilings[k], NULL, &depth_unit);
-					put_format(b, translate("gettextFromC", "Tissue %.0fmin: %.0f%s\n"), buehlmann_N2_t_halflife[k], depthvalue, depth_unit);
+					put_format(b, translate("gettextFromC", "Tissue %.0fmin: %.1f%s\n"), buehlmann_N2_t_halflife[k], depthvalue, depth_unit);
 				}
 			}
 		}
