@@ -8,6 +8,7 @@
 #include "configuredivecomputer.h"
 #include <QStyledItemDelegate>
 #include <QNetworkAccessManager>
+#include "btdeviceselectiondialog.h"
 
 class GasSpinBoxItemDelegate : public QStyledItemDelegate {
 	Q_OBJECT
@@ -16,6 +17,7 @@ public:
 	enum column_type {
 		PERCENT,
 		DEPTH,
+		SETPOINT,
 	};
 
 	GasSpinBoxItemDelegate(QObject *parent = 0, column_type type = PERCENT);
@@ -56,8 +58,13 @@ public:
 	explicit ConfigureDiveComputerDialog(QWidget *parent = 0);
 	~ConfigureDiveComputerDialog();
 
+protected:
+	void closeEvent(QCloseEvent *event);
+
 private
 slots:
+	void checkLogFile(int state);
+	void pickLogFile();
 	void readSettings();
 	void resetSettings();
 	void configMessage(QString msg);
@@ -75,9 +82,19 @@ slots:
 
 	void on_DiveComputerList_currentRowChanged(int currentRow);
 
+	void dc_open();
+	void dc_close();
+
+#if BT_SUPPORT
+	void bluetoothSelectionDialogIsFinished(int result);
+	void selectRemoteBluetoothDevice();
+#endif
+
 private:
 	Ui::ConfigureDiveComputerDialog ui;
 
+	QString logFile;
+
 	QStringList vendorList;
 	QHash<QString, QStringList> productList;
 
@@ -100,6 +117,9 @@ private:
 
 	QString selected_vendor;
 	QString selected_product;
+	bool fw_upgrade_possible;
+
+	BtDeviceSelectionDialog *btDeviceSelectionDialog;
 };
 
 class OstcFirmwareCheck : QObject {
