@@ -1,5 +1,6 @@
 #include "divelogimportdialog.h"
 #include "mainwindow.h"
+#include "color.h"
 #include "ui_divelogimportdialog.h"
 #include <QShortcut>
 #include <QDrag>
@@ -9,23 +10,36 @@ static QString subsurface_mimedata = "subsurface/csvcolumns";
 static QString subsurface_index = "subsurface/csvindex";
 
 const DiveLogImportDialog::CSVAppConfig DiveLogImportDialog::CSVApps[CSVAPPS] = {
-	// time, depth, temperature, po2, cns, ndl, tts, stopdepth, pressure
+	// time, depth, temperature, po2, sensor1, sensor2, sensor3, cns, ndl, tts, stopdepth, pressure, setpoint
 	// indices are 0 based, -1 means the column doesn't exist
 	{ "Manual import", },
-	{ "APD Log Viewer", 0, 1, 15, 6, 17, -1, -1, 18, -1, "Tab" },
-	{ "XP5", 0, 1, 9, -1, -1, -1, -1, -1, -1, "Tab" },
-	{ "SensusCSV", 9, 10, -1, -1, -1, -1, -1, -1, -1, "," },
-	{ "Seabear CSV", 0, 1, 5, -1, -1, 2, 3, 4, 6, ";" },
-	{ "SubsurfaceCSV", -1, -1, -1, -1, -1, -1, -1, -1, -1, "Tab" },
+	{ "APD Log Viewer - DC1", 0, 1, 15, 6, 3, 4, 5, 17, -1, -1, 18, -1, 2, "Tab" },
+	{ "APD Log Viewer - DC2", 0, 1, 15, 6, 7, 8, 9, 17, -1, -1, 18, -1, 2, "Tab" },
+	{ "XP5", 0, 1, 9, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, "Tab" },
+	{ "SensusCSV", 9, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, "," },
+	{ "Seabear CSV", 0, 1, 5, -1, -1, -1, -1, -1, 2, 3, 4, 6, -1, ";" },
+	{ "SubsurfaceCSV", -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, "Tab" },
 	{ NULL, }
 };
 
+static enum {
+	MANUAL,
+	APD,
+	APD2,
+	XP5,
+	SENSUS,
+	SEABEAR,
+	SUBSURFACE
+} known;
+
 ColumnNameProvider::ColumnNameProvider(QObject *parent) : QAbstractListModel(parent)
 {
 	columnNames << tr("Dive #") << tr("Date") << tr("Time") << tr("Duration") << tr("Location") << tr("GPS") << tr("Weight") << tr("Cyl. size") << tr("Start pressure") <<
-		       tr("End pressure") << tr("Max. depth") << tr("Avg. depth") << tr("Divemaster") << tr("Buddy") << tr("Notes") << tr("Tags") << tr("Air temp.") << tr("Water temp.") <<
+		       tr("End pressure") << tr("Max. depth") << tr("Avg. depth") << tr("Divemaster") << tr("Buddy") << tr("Suit") << tr("Notes") << tr("Tags") << tr("Air temp.") << tr("Water temp.") <<
 		       tr("O₂") << tr("He") << tr("Sample time") << tr("Sample depth") << tr("Sample temperature") << tr("Sample pO₂") << tr("Sample CNS") << tr("Sample NDL") <<
-		       tr("Sample TTS") << tr("Sample stopdepth") << tr("Sample pressure");
+		       tr("Sample TTS") << tr("Sample stopdepth") << tr("Sample pressure") <<
+		       tr("Sample sensor1 pO₂") << tr("Sample sensor2 pO₂") << tr("Sample sensor3 pO₂") <<
+		       tr("Sample setpoint");
 }
 
 bool ColumnNameProvider::insertRows(int row, int count, const QModelIndex &parent)
@@ -315,10 +329,12 @@ DiveLogImportDialog::DiveLogImportDialog(QStringList fn, QWidget *parent) : QDia
 	ui->setupUi(this);
 	fileNames = fn;
 	column = 0;
+	delta = "0";
+	hw = "";
 
 	/* Add indexes of XSLTs requiring special handling to the list */
-	specialCSV << 3;
-	specialCSV << 5;
+	specialCSV << SENSUS;
+	specialCSV << SUBSURFACE;
 
 	for (int i = 0; !CSVApps[i].name.isNull(); ++i)
 		ui->knownImports->addItem(CSVApps[i].name);
@@ -374,7 +390,65 @@ void DiveLogImportDialog::loadFileContents(int value, whatChanged triggeredBy)
 	QString firstLine = f.readLine();
 	if (firstLine.contains("SEABEAR")) {
 		seabear = true;
-		firstLine = "Sample time;Sample depth;Sample NDL;Sample TTS;Sample stopdepth;Sample temperature;Sample pressure";
+
+		/*
+		 * Parse header - currently only interested in sample
+		 * interval and hardware version. If we have old format
+		 * the interval value is missing from the header.
+		 */
+
+		while ((firstLine = f.readLine().trimmed()).length() > 0 && !f.atEnd()) {
+			if (firstLine.contains("//Hardware Version: ")) {
+				hw = firstLine.replace(QString::fromLatin1("//Hardware Version: "), QString::fromLatin1("\"Seabear ")).trimmed().append("\"");
+				break;
+			}
+		}
+
+		/*
+		 * Note that we scan over the "Log interval" on purpose
+		 */
+
+		while ((firstLine = f.readLine().trimmed()).length() > 0 && !f.atEnd()) {
+			if (firstLine.contains("//Log interval: "))
+				delta = firstLine.remove(QString::fromLatin1("//Log interval: ")).trimmed().remove(QString::fromLatin1(" s"));
+		}
+
+		/*
+		 * Parse CSV fields
+		 */
+
+		firstLine = f.readLine().trimmed();
+
+		currColumns = firstLine.split(';');
+		Q_FOREACH (QString columnText, currColumns) {
+			if (columnText == "Time") {
+				headers.append("Sample time");
+			} else if (columnText == "Depth") {
+				headers.append("Sample depth");
+			} else if (columnText == "Temperature") {
+				headers.append("Sample temperature");
+			} else if (columnText == "NDT") {
+				headers.append("Sample NDL");
+			} else if (columnText == "TTS") {
+				headers.append("Sample TTS");
+			} else if (columnText == "pO2_1") {
+				headers.append("Sample sensor1 pO₂");
+			} else if (columnText == "pO2_2") {
+				headers.append("Sample sensor2 pO₂");
+			} else if (columnText == "pO2_3") {
+				headers.append("Sample sensor3 pO₂");
+			} else if (columnText == "Ceiling") {
+				headers.append("Sample ceiling");
+			} else if (columnText == "Tank pressure") {
+				headers.append("Sample pressure");
+			} else {
+				// We do not know about this value
+				qDebug() << "Seabear import found an un-handled field: " << columnText;
+				headers.append("");
+			}
+		}
+
+		firstLine = headers.join(";");
 		blockSignals(true);
 		ui->knownImports->setCurrentText("Seabear CSV");
 		blockSignals(false);
@@ -388,13 +462,13 @@ void DiveLogImportDialog::loadFileContents(int value, whatChanged triggeredBy)
 	}
 
 	// Special handling for APD Log Viewer
-	if ((triggeredBy == KNOWNTYPES && value == 1) || (triggeredBy == INITIAL && fileNames.first().endsWith(".apd", Qt::CaseInsensitive))) {
+	if ((triggeredBy == KNOWNTYPES && (value == APD || value == APD2)) || (triggeredBy == INITIAL && fileNames.first().endsWith(".apd", Qt::CaseInsensitive))) {
 		apd=true;
-		firstLine = "Sample time\tSample depth\t\t\t\t\tSample pO₂\t\t\t\t\t\t\t\t\tSample temperature\t\tSample CNS\tSample stopdepth";
+		firstLine = "Sample time\tSample depth\tSample setpoint\tSample sensor1 pO₂\tSample sensor2 pO₂\tSample sensor3 pO₂\tSample pO₂\t\t\t\t\t\t\t\t\tSample temperature\t\tSample CNS\tSample stopdepth";
 		blockSignals(true);
 		ui->CSVSeparator->setCurrentText(tr("Tab"));
 		if (triggeredBy == INITIAL && fileNames.first().contains(".apd", Qt::CaseInsensitive))
-			ui->knownImports->setCurrentText("APD Log Viewer");
+			ui->knownImports->setCurrentText("APD Log Viewer - DC1");
 		blockSignals(false);
 	}
 
@@ -418,14 +492,22 @@ void DiveLogImportDialog::loadFileContents(int value, whatChanged triggeredBy)
 			currColumns = firstLine.split(separator);
 		}
 	}
-	if (triggeredBy == INITIAL || (triggeredBy == KNOWNTYPES && value == 0) || triggeredBy == SEPARATOR) {
+	if (triggeredBy == INITIAL || (triggeredBy == KNOWNTYPES && value == MANUAL) || triggeredBy == SEPARATOR) {
 		// now try and guess the columns
 		Q_FOREACH (QString columnText, currColumns) {
-			columnText.replace("\"", "");
-			columnText.replace("number", "#", Qt::CaseInsensitive);
-			columnText.replace("2", "₂", Qt::CaseInsensitive);
-			columnText.replace("cylinder", "cyl.", Qt::CaseInsensitive);
-			int idx = provider->mymatch(columnText);
+			/*
+			 * We have to skip the conversion of 2 to ₂ for APD Log
+			 * viewer as that would mess up the sensor numbering. We
+			 * also know that the column headers do not need this
+			 * conversion.
+			 */
+			if (apd == false) {
+				columnText.replace("\"", "");
+				columnText.replace("number", "#", Qt::CaseInsensitive);
+				columnText.replace("2", "₂", Qt::CaseInsensitive);
+				columnText.replace("cylinder", "cyl.", Qt::CaseInsensitive);
+			}
+			int idx = provider->mymatch(columnText.trimmed());
 			if (idx >= 0) {
 				QString foundHeading = provider->data(provider->index(idx, 0), Qt::DisplayRole).toString();
 				provider->removeRow(idx);
@@ -444,9 +526,9 @@ void DiveLogImportDialog::loadFileContents(int value, whatChanged triggeredBy)
 			}
 		}
 	}
-	if (triggeredBy == KNOWNTYPES && value != 0) {
+	if (triggeredBy == KNOWNTYPES && value != MANUAL) {
 		// an actual known type
-		if (value == 5) {
+		if (value == SUBSURFACE) {
 			/*
 			 * Subsurface CSV file needs separator detection
 			 * as we used to default to comma but switched
@@ -470,7 +552,7 @@ void DiveLogImportDialog::loadFileContents(int value, whatChanged triggeredBy)
 				separator = "\t";
 			currColumns = firstLine.split(separator);
 		}
-		// now set up time, depth, temperature, po2, cns, ndl, tts, stopdepth, pressure
+		// now set up time, depth, temperature, po2, cns, ndl, tts, stopdepth, pressure, setpoint
 		for (int i = 0; i < currColumns.count(); i++)
 			headers.append("");
 		if (CSVApps[value].time > -1 && CSVApps[value].time < currColumns.count())
@@ -481,6 +563,12 @@ void DiveLogImportDialog::loadFileContents(int value, whatChanged triggeredBy)
 			headers.replace(CSVApps[value].temperature, tr("Sample temperature"));
 		if (CSVApps[value].po2 > -1 && CSVApps[value].po2 < currColumns.count())
 			headers.replace(CSVApps[value].po2, tr("Sample pO₂"));
+		if (CSVApps[value].sensor1 > -1 && CSVApps[value].sensor1 < currColumns.count())
+			headers.replace(CSVApps[value].sensor1, tr("Sample sensor1 pO₂"));
+		if (CSVApps[value].sensor2 > -1 && CSVApps[value].sensor2 < currColumns.count())
+			headers.replace(CSVApps[value].sensor2, tr("Sample sensor2 pO₂"));
+		if (CSVApps[value].sensor3 > -1 && CSVApps[value].sensor3 < currColumns.count())
+			headers.replace(CSVApps[value].sensor3, tr("Sample sensor3 pO₂"));
 		if (CSVApps[value].cns > -1 && CSVApps[value].cns < currColumns.count())
 			headers.replace(CSVApps[value].cns, tr("Sample CNS"));
 		if (CSVApps[value].ndl > -1 && CSVApps[value].ndl < currColumns.count())
@@ -491,6 +579,41 @@ void DiveLogImportDialog::loadFileContents(int value, whatChanged triggeredBy)
 			headers.replace(CSVApps[value].stopdepth, tr("Sample stopdepth"));
 		if (CSVApps[value].pressure > -1 && CSVApps[value].pressure < currColumns.count())
 			headers.replace(CSVApps[value].pressure, tr("Sample pressure"));
+		if (CSVApps[value].setpoint > -1 && CSVApps[value].setpoint < currColumns.count())
+			headers.replace(CSVApps[value].setpoint, tr("Sample setpoint"));
+
+		/* Show the Subsurface CSV column headers */
+		if (value == SUBSURFACE && currColumns.count() >= 23) {
+			headers.replace(0, tr("Dive #"));
+			headers.replace(1, tr("Date"));
+			headers.replace(2, tr("Time"));
+			headers.replace(3, tr("Duration"));
+			headers.replace(4, tr("Max. depth"));
+			headers.replace(5, tr("Avg. depth"));
+			headers.replace(6, tr("Air temp."));
+			headers.replace(7, tr("Water temp."));
+			headers.replace(8, tr("Cyl. size"));
+			headers.replace(9, tr("Start pressure"));
+			headers.replace(10, tr("End pressure"));
+			headers.replace(11, tr("O₂"));
+			headers.replace(12, tr("He"));
+			headers.replace(13, tr("Location"));
+			headers.replace(14, tr("GPS"));
+			headers.replace(15, tr("Divemaster"));
+			headers.replace(16, tr("Buddy"));
+			headers.replace(17, tr("Suit"));
+			headers.replace(18, tr("Rating"));
+			headers.replace(19, tr("Visibility"));
+			headers.replace(20, tr("Notes"));
+			headers.replace(21, tr("Weight"));
+			headers.replace(22, tr("Tags"));
+
+			blockSignals(true);
+			ui->CSVSeparator->setCurrentText(separator);
+			ui->DateFormat->setCurrentText("yyyy-mm-dd");
+			ui->DurationFormat->setCurrentText("Minutes:seconds");
+			blockSignals(false);
+		}
 	}
 
 	f.reset();
@@ -513,7 +636,7 @@ void DiveLogImportDialog::loadFileContents(int value, whatChanged triggeredBy)
 	}
 
 	while (rows < 10 && !f.atEnd()) {
-		QString currLine = f.readLine();
+		QString currLine = f.readLine().trimmed();
 		currColumns = currLine.split(separator);
 		fileColumns.append(currColumns);
 		rows += 1;
@@ -524,28 +647,110 @@ void DiveLogImportDialog::loadFileContents(int value, whatChanged triggeredBy)
 			resultModel->setData(resultModel->index(0, i),headers.at(i),Qt::EditRole);
 }
 
+char *intdup(int index)
+{
+	char tmpbuf[21];
+
+	snprintf(tmpbuf, sizeof(tmpbuf) - 2, "%d", index);
+	tmpbuf[20] = 0;
+	return strdup(tmpbuf);
+}
+
+int DiveLogImportDialog::setup_csv_params(QStringList r, char **params, int pnr)
+{
+	params[pnr++] = strdup("timeField");
+	params[pnr++] = intdup(r.indexOf(tr("Sample time")));
+	params[pnr++] = strdup("depthField");
+	params[pnr++] = intdup(r.indexOf(tr("Sample depth")));
+	params[pnr++] = strdup("tempField");
+	params[pnr++] = intdup(r.indexOf(tr("Sample temperature")));
+	params[pnr++] = strdup("po2Field");
+	params[pnr++] = intdup(r.indexOf(tr("Sample pO₂")));
+	params[pnr++] = strdup("o2sensor1Field");
+	params[pnr++] = intdup(r.indexOf(tr("Sample sensor1 pO₂")));
+	params[pnr++] = strdup("o2sensor2Field");
+	params[pnr++] = intdup(r.indexOf(tr("Sample sensor2 pO₂")));
+	params[pnr++] = strdup("o2sensor3Field");
+	params[pnr++] = intdup(r.indexOf(tr("Sample sensor3 pO₂")));
+	params[pnr++] = strdup("cnsField");
+	params[pnr++] = intdup(r.indexOf(tr("Sample CNS")));
+	params[pnr++] = strdup("ndlField");
+	params[pnr++] = intdup(r.indexOf(tr("Sample NDL")));
+	params[pnr++] = strdup("ttsField");
+	params[pnr++] = intdup(r.indexOf(tr("Sample TTS")));
+	params[pnr++] = strdup("stopdepthField");
+	params[pnr++] = intdup(r.indexOf(tr("Sample stopdepth")));
+	params[pnr++] = strdup("pressureField");
+	params[pnr++] = intdup(r.indexOf(tr("Sample pressure")));
+	params[pnr++] = strdup("setpointFiend");
+	params[pnr++] = intdup(r.indexOf(tr("Sample setpoint")));
+	params[pnr++] = strdup("separatorIndex");
+	params[pnr++] = intdup(ui->CSVSeparator->currentIndex());
+	params[pnr++] = strdup("units");
+	params[pnr++] = intdup(ui->CSVUnits->currentIndex());
+	if (hw.length()) {
+		params[pnr++] = strdup("hw");
+		params[pnr++] = strdup(hw.toUtf8().data());
+	} else if (ui->knownImports->currentText().length() > 0) {
+		params[pnr++] = strdup("hw");
+		params[pnr++] = strdup(ui->knownImports->currentText().prepend("\"").append("\"").toUtf8().data());
+	}
+	params[pnr++] = NULL;
+
+	return pnr;
+}
+
 void DiveLogImportDialog::on_buttonBox_accepted()
 {
 	QStringList r = resultModel->result();
 	if (ui->knownImports->currentText() != "Manual import") {
 		for (int i = 0; i < fileNames.size(); ++i) {
 			if (ui->knownImports->currentText() == "Seabear CSV") {
-				if (parse_seabear_csv_file(fileNames[i].toUtf8().data(),
-						       r.indexOf(tr("Sample time")),
-						       r.indexOf(tr("Sample depth")),
-						       r.indexOf(tr("Sample temperature")),
-						       r.indexOf(tr("Sample pO₂")),
-						       r.indexOf(tr("Sample CNS")),
-						       r.indexOf(tr("Sample NDL")),
-						       r.indexOf(tr("Sample TTS")),
-						       r.indexOf(tr("Sample stopdepth")),
-						       r.indexOf(tr("Sample pressure")),
-						       ui->CSVSeparator->currentIndex(),
-						       specialCSV.contains(ui->knownImports->currentIndex()) ? CSVApps[ui->knownImports->currentIndex()].name.toUtf8().data() : "csv",
-						       ui->CSVUnits->currentIndex()
-						       ) < 0)
-					return;
+				char *params[40];
+				int pnr = 0;
 
+				params[pnr++] = strdup("timeField");
+				params[pnr++] = intdup(r.indexOf(tr("Sample time")));
+				params[pnr++] = strdup("depthField");
+				params[pnr++] = intdup(r.indexOf(tr("Sample depth")));
+				params[pnr++] = strdup("tempField");
+				params[pnr++] = intdup(r.indexOf(tr("Sample temperature")));
+				params[pnr++] = strdup("po2Field");
+				params[pnr++] = intdup(r.indexOf(tr("Sample pO₂")));
+				params[pnr++] = strdup("o2sensor1Field");
+				params[pnr++] = intdup(r.indexOf(tr("Sample sensor1 pO₂")));
+				params[pnr++] = strdup("o2sensor2Field");
+				params[pnr++] = intdup(r.indexOf(tr("Sample sensor2 pO₂")));
+				params[pnr++] = strdup("o2sensor3Field");
+				params[pnr++] = intdup(r.indexOf(tr("Sample sensor3 pO₂")));
+				params[pnr++] = strdup("cnsField");
+				params[pnr++] = intdup(r.indexOf(tr("Sample CNS")));
+				params[pnr++] = strdup("ndlField");
+				params[pnr++] = intdup(r.indexOf(tr("Sample NDL")));
+				params[pnr++] = strdup("ttsField");
+				params[pnr++] = intdup(r.indexOf(tr("Sample TTS")));
+				params[pnr++] = strdup("stopdepthField");
+				params[pnr++] = intdup(r.indexOf(tr("Sample stopdepth")));
+				params[pnr++] = strdup("pressureField");
+				params[pnr++] = intdup(r.indexOf(tr("Sample pressure")));
+				params[pnr++] = strdup("setpointFiend");
+				params[pnr++] = intdup(-1);
+				params[pnr++] = strdup("separatorIndex");
+				params[pnr++] = intdup(ui->CSVSeparator->currentIndex());
+				params[pnr++] = strdup("units");
+				params[pnr++] = intdup(ui->CSVUnits->currentIndex());
+				params[pnr++] = strdup("delta");
+				params[pnr++] = strdup(delta.toUtf8().data());
+				if (hw.length()) {
+					params[pnr++] = strdup("hw");
+					params[pnr++] = strdup(hw.toUtf8().data());
+				}
+				params[pnr++] = NULL;
+
+				if (parse_seabear_csv_file(fileNames[i].toUtf8().data(),
+							params, pnr - 1, "csv") < 0) {
+					return;
+				}
 				// Seabear CSV stores NDL and TTS in Minutes, not seconds
 				struct dive *dive = dive_table.dives[dive_table.nr - 1];
 				for(int s_nr = 0 ; s_nr <= dive->dc.samples ; s_nr++) {
@@ -554,68 +759,83 @@ void DiveLogImportDialog::on_buttonBox_accepted()
 					sample->tts.seconds *= 60;
 				}
 			} else {
-				parse_csv_file(fileNames[i].toUtf8().data(),
-					       r.indexOf(tr("Sample time")),
-					       r.indexOf(tr("Sample depth")),
-					       r.indexOf(tr("Sample temperature")),
-					       r.indexOf(tr("Sample pO₂")),
-					       r.indexOf(tr("Sample CNS")),
-					       r.indexOf(tr("Sample NDL")),
-					       r.indexOf(tr("Sample TTS")),
-					       r.indexOf(tr("Sample stopdepth")),
-					       r.indexOf(tr("Sample pressure")),
-					       ui->CSVSeparator->currentIndex(),
-					       specialCSV.contains(ui->knownImports->currentIndex()) ? CSVApps[ui->knownImports->currentIndex()].name.toUtf8().data() : "csv",
-					       ui->CSVUnits->currentIndex()
-					       );
+				char *params[37];
+				int pnr = 0;
+
+				pnr = setup_csv_params(r, params, pnr);
+				parse_csv_file(fileNames[i].toUtf8().data(), params, pnr - 1,
+						specialCSV.contains(ui->knownImports->currentIndex()) ? CSVApps[ui->knownImports->currentIndex()].name.toUtf8().data() : "csv");
 			}
 		}
 	} else {
 		for (int i = 0; i < fileNames.size(); ++i) {
-			if (r.indexOf(tr("Sample time")) < 0)
-				parse_manual_file(fileNames[i].toUtf8().data(),
-						ui->CSVSeparator->currentIndex(),
-						ui->CSVUnits->currentIndex(),
-						ui->DateFormat->currentIndex(),
-						ui->DurationFormat->currentIndex(),
-						r.indexOf(tr("Dive #")),
-						r.indexOf(tr("Date")),
-						r.indexOf(tr("Time")),
-						r.indexOf(tr("Duration")),
-						r.indexOf(tr("Location")),
-						r.indexOf(tr("GPS")),
-						r.indexOf(tr("Max. depth")),
-						r.indexOf(tr("Mean depth")),
-						r.indexOf(tr("Divemaster")),
-						r.indexOf(tr("Buddy")),
-						r.indexOf(tr("Notes")),
-						r.indexOf(tr("Weight")),
-						r.indexOf(tr("Tags")),
-						r.indexOf(tr("Cyl. size")),
-						r.indexOf(tr("Start pressure")),
-						r.indexOf(tr("End pressure")),
-						r.indexOf(tr("O₂")),
-						r.indexOf(tr("He")),
-						r.indexOf(tr("Air temp.")),
-						r.indexOf(tr("Water temp."))
-							);
-			else
-				parse_csv_file(fileNames[i].toUtf8().data(),
-					       r.indexOf(tr("Sample time")),
-					       r.indexOf(tr("Sample depth")),
-					       r.indexOf(tr("Sample temperature")),
-					       r.indexOf(tr("Sample pO₂")),
-					       r.indexOf(tr("Sample CNS")),
-					       r.indexOf(tr("Sample NDL")),
-					       r.indexOf(tr("Sample TTS")),
-					       r.indexOf(tr("Sample stopdepth")),
-					       r.indexOf(tr("Sample pressure")),
-					       ui->CSVSeparator->currentIndex(),
-					       specialCSV.contains(ui->knownImports->currentIndex()) ? CSVApps[ui->knownImports->currentIndex()].name.toUtf8().data() : "csv",
-					       ui->CSVUnits->currentIndex()
-					       );
+			if (r.indexOf(tr("Sample time")) < 0) {
+				char *params[55];
+				int pnr = 0;
+				params[pnr++] = strdup("numberField");
+				params[pnr++] = intdup(r.indexOf(tr("Dive #")));
+				params[pnr++] = strdup("dateField");
+				params[pnr++] = intdup(r.indexOf(tr("Date")));
+				params[pnr++] = strdup("timeField");
+				params[pnr++] = intdup(r.indexOf(tr("Time")));
+				params[pnr++] = strdup("durationField");
+				params[pnr++] = intdup(r.indexOf(tr("Duration")));
+				params[pnr++] = strdup("locationField");
+				params[pnr++] = intdup(r.indexOf(tr("Location")));
+				params[pnr++] = strdup("gpsField");
+				params[pnr++] = intdup(r.indexOf(tr("GPS")));
+				params[pnr++] = strdup("maxDepthField");
+				params[pnr++] = intdup(r.indexOf(tr("Max. depth")));
+				params[pnr++] = strdup("meanDepthField");
+				params[pnr++] = intdup(r.indexOf(tr("Avg. depth")));
+				params[pnr++] = strdup("divemasterField");
+				params[pnr++] = intdup(r.indexOf(tr("Divemaster")));
+				params[pnr++] = strdup("buddyField");
+				params[pnr++] = intdup(r.indexOf(tr("Buddy")));
+				params[pnr++] = strdup("suitField");
+				params[pnr++] = intdup(r.indexOf(tr("Suit")));
+				params[pnr++] = strdup("notesField");
+				params[pnr++] = intdup(r.indexOf(tr("Notes")));
+				params[pnr++] = strdup("weightField");
+				params[pnr++] = intdup(r.indexOf(tr("Weight")));
+				params[pnr++] = strdup("tagsField");
+				params[pnr++] = intdup(r.indexOf(tr("Tags")));
+				params[pnr++] = strdup("separatorIndex");
+				params[pnr++] = intdup(ui->CSVSeparator->currentIndex());
+				params[pnr++] = strdup("units");
+				params[pnr++] = intdup(ui->CSVUnits->currentIndex());
+				params[pnr++] = strdup("datefmt");
+				params[pnr++] = intdup(ui->DateFormat->currentIndex());
+				params[pnr++] = strdup("durationfmt");
+				params[pnr++] = intdup(ui->DurationFormat->currentIndex());
+				params[pnr++] = strdup("cylindersizeField");
+				params[pnr++] = intdup(r.indexOf(tr("Cyl. size")));
+				params[pnr++] = strdup("startpressureField");
+				params[pnr++] = intdup(r.indexOf(tr("Start pressure")));
+				params[pnr++] = strdup("endpressureField");
+				params[pnr++] = intdup(r.indexOf(tr("End pressure")));
+				params[pnr++] = strdup("o2Field");
+				params[pnr++] = intdup(r.indexOf(tr("O₂")));
+				params[pnr++] = strdup("heField");
+				params[pnr++] = intdup(r.indexOf(tr("He")));
+				params[pnr++] = strdup("airtempField");
+				params[pnr++] = intdup(r.indexOf(tr("Air temp.")));
+				params[pnr++] = strdup("watertempField");
+				params[pnr++] = intdup(r.indexOf(tr("Water temp.")));
+				params[pnr++] = NULL;
+
+				parse_manual_file(fileNames[i].toUtf8().data(), params, pnr - 1);
+			} else {
+				char *params[37];
+				int pnr = 0;
+
+				pnr = setup_csv_params(r, params, pnr);
+				parse_csv_file(fileNames[i].toUtf8().data(), params, pnr - 1,
+						specialCSV.contains(ui->knownImports->currentIndex()) ? CSVApps[ui->knownImports->currentIndex()].name.toUtf8().data() : "csv");
+			}
 		}
 	}
+
 	process_dives(true, false);
 	MainWindow::instance()->refreshDisplay();
 }
