@@ -6,13 +6,24 @@
  */
 #include "maintab.h"
 #include "mainwindow.h"
+#include "globe.h"
 #include "helpers.h"
 #include "statistics.h"
 #include "modeldelegates.h"
-
-#if defined(FBSUPPORT)
-#include "socialnetworks.h"
-#endif
+#include "diveplannermodel.h"
+#include "divelistview.h"
+#include "display.h"
+#include "profile/profilewidget2.h"
+#include "diveplanner.h"
+#include "divesitehelpers.h"
+#include "cylindermodel.h"
+#include "weightmodel.h"
+#include "divepicturemodel.h"
+#include "divecomputerextradatamodel.h"
+#include "divelocationmodel.h"
+#include "divesite.h"
+#include "locationinformation.h"
+#include "divesite.h"
 
 #include <QCompleter>
 #include <QSettings>
@@ -20,6 +31,7 @@
 #include <QShortcut>
 #include <QMessageBox>
 #include <QDesktopServices>
+#include <QStringList>
 
 MainTab::MainTab(QWidget *parent) : QTabWidget(parent),
 	weightModel(new WeightModel(this)),
@@ -43,18 +55,23 @@ MainTab::MainTab(QWidget *parent) : QTabWidget(parent),
 	ui.extraData->setModel(extraDataModel);
 	closeMessage();
 
+	connect(ui.editDiveSiteButton, SIGNAL(clicked()), MainWindow::instance(), SIGNAL(startDiveSiteEdit()));
+#ifndef NO_MARBLE
+	connect(ui.location, &DiveLocationLineEdit::entered, GlobeGPS::instance(), &GlobeGPS::centerOnIndex);
+	connect(ui.location, &DiveLocationLineEdit::currentChanged, GlobeGPS::instance(), &GlobeGPS::centerOnIndex);
+#endif
+
 	QAction *action = new QAction(tr("Apply changes"), this);
 	connect(action, SIGNAL(triggered(bool)), this, SLOT(acceptChanges()));
 	addMessageAction(action);
 
 	action = new QAction(tr("Discard changes"), this);
 	connect(action, SIGNAL(triggered(bool)), this, SLOT(rejectChanges()));
+	addMessageAction(action);
 
 	QShortcut *closeKey = new QShortcut(QKeySequence(Qt::Key_Escape), this);
 	connect(closeKey, SIGNAL(activated()), this, SLOT(escDetected()));
 
-	addMessageAction(action);
-
 	if (qApp->style()->objectName() == "oxygen")
 		setDocumentMode(true);
 	else
@@ -64,21 +81,6 @@ MainTab::MainTab(QWidget *parent) : QTabWidget(parent),
 	// filled from a dive, they are made writeable
 	setEnabled(false);
 
-	ui.location->installEventFilter(this);
-	ui.coordinates->installEventFilter(this);
-	ui.divemaster->installEventFilter(this);
-	ui.buddy->installEventFilter(this);
-	ui.suit->installEventFilter(this);
-	ui.notes->viewport()->installEventFilter(this);
-	ui.rating->installEventFilter(this);
-	ui.visibility->installEventFilter(this);
-	ui.airtemp->installEventFilter(this);
-	ui.watertemp->installEventFilter(this);
-	ui.dateEdit->installEventFilter(this);
-	ui.timeEdit->installEventFilter(this);
-	ui.tagWidget->installEventFilter(this);
-	ui.DiveType->installEventFilter(this);
-
 	Q_FOREACH (QObject *obj, ui.statisticsTab->children()) {
 		QLabel *label = qobject_cast<QLabel *>(obj);
 		if (label)
@@ -105,20 +107,20 @@ MainTab::MainTab(QWidget *parent) : QTabWidget(parent),
 	ui.cylinders->view()->setColumnHidden(CylindersModel::DEPTH, true);
 	completers.buddy = new QCompleter(&buddyModel, ui.buddy);
 	completers.divemaster = new QCompleter(&diveMasterModel, ui.divemaster);
-	completers.location = new QCompleter(&locationModel, ui.location);
 	completers.suit = new QCompleter(&suitModel, ui.suit);
 	completers.tags = new QCompleter(&tagModel, ui.tagWidget);
 	completers.buddy->setCaseSensitivity(Qt::CaseInsensitive);
 	completers.divemaster->setCaseSensitivity(Qt::CaseInsensitive);
-	completers.location->setCaseSensitivity(Qt::CaseInsensitive);
 	completers.suit->setCaseSensitivity(Qt::CaseInsensitive);
 	completers.tags->setCaseSensitivity(Qt::CaseInsensitive);
 	ui.buddy->setCompleter(completers.buddy);
 	ui.divemaster->setCompleter(completers.divemaster);
-	ui.location->setCompleter(completers.location);
 	ui.suit->setCompleter(completers.suit);
 	ui.tagWidget->setCompleter(completers.tags);
-
+	ui.diveNotesMessage->hide();
+	ui.diveEquipmentMessage->hide();
+	ui.diveInfoMessage->hide();
+	ui.diveStatisticsMessage->hide();
 	setMinimumHeight(0);
 	setMinimumWidth(0);
 
@@ -158,6 +160,7 @@ MainTab::MainTab(QWidget *parent) : QTabWidget(parent),
 		label->setContentsMargins(margins);
 	}
 	ui.cylinders->view()->horizontalHeader()->setContextMenuPolicy(Qt::ActionsContextMenu);
+	ui.weights->view()->horizontalHeader()->setContextMenuPolicy(Qt::ActionsContextMenu);
 
 	QSettings s;
 	s.beginGroup("cylinders_dialog");
@@ -181,18 +184,27 @@ MainTab::MainTab(QWidget *parent) : QTabWidget(parent),
 	ui.photosView->setSelectionMode(QAbstractItemView::SingleSelection);
 	connect(deletePhoto, SIGNAL(triggered(bool)), this, SLOT(removeSelectedPhotos()));
 
-#if defined(FBSUPPORT)
-	FacebookManager *fb = FacebookManager::instance();
-	connect(fb, &FacebookManager::justLoggedIn, ui.facebookPublish, &QPushButton::show);
-	connect(fb, &FacebookManager::justLoggedOut, ui.facebookPublish, &QPushButton::hide);
-	connect(ui.facebookPublish, &QPushButton::clicked, fb, &FacebookManager::sendDive);
-	ui.facebookPublish->setVisible(fb->loggedIn());
-#else
-	ui.facebookPublish->setVisible(false);
-	ui.socialNetworks->setVisible(false);
-#endif
+	ui.waitingSpinner->setRoundness(70.0);
+	ui.waitingSpinner->setMinimumTrailOpacity(15.0);
+	ui.waitingSpinner->setTrailFadePercentage(70.0);
+	ui.waitingSpinner->setNumberOfLines(8);
+	ui.waitingSpinner->setLineLength(5);
+	ui.waitingSpinner->setLineWidth(3);
+	ui.waitingSpinner->setInnerRadius(5);
+	ui.waitingSpinner->setRevolutionsPerSecond(1);
+
+	connect(ReverseGeoLookupThread::instance(), SIGNAL(finished()),
+			LocationInformationModel::instance(), SLOT(update()));
+
+	connect(ReverseGeoLookupThread::instance(), &QThread::finished,
+			this, &MainTab::setCurrentLocationIndex);
+
+	connect(ui.diveNotesMessage, &KMessageWidget::showAnimationFinished,
+					ui.location, &DiveLocationLineEdit::fixPopupPosition);
 
 	acceptingEdit = false;
+
+	ui.diveTripLocation->hide();
 }
 
 MainTab::~MainTab()
@@ -206,6 +218,27 @@ MainTab::~MainTab()
 	}
 }
 
+void MainTab::setCurrentLocationIndex()
+{
+	if (current_dive) {
+		struct dive_site *ds = get_dive_site_by_uuid(current_dive->dive_site_uuid);
+		if (ds)
+			ui.location->setCurrentDiveSiteUuid(ds->uuid);
+		else
+			ui.location->clear();
+	}
+}
+
+void MainTab::enableGeoLookupEdition()
+{
+	ui.waitingSpinner->stop();
+}
+
+void MainTab::disableGeoLookupEdition()
+{
+	ui.waitingSpinner->start();
+}
+
 void MainTab::toggleTriggeredColumn()
 {
 	QAction *action = qobject_cast<QAction *>(sender());
@@ -266,9 +299,6 @@ void MainTab::displayMessage(QString str)
 	ui.diveStatisticsMessage->setText(str);
 	ui.diveStatisticsMessage->animatedShow();
 	updateTextLabels();
-	ui.tagWidget->fixPopupPosition(ui.diveNotesMessage->bestContentHeight());
-	ui.buddy->fixPopupPosition(ui.diveNotesMessage->bestContentHeight());
-	ui.divemaster->fixPopupPosition(ui.diveNotesMessage->bestContentHeight());
 }
 
 void MainTab::updateTextLabels(bool showUnits)
@@ -284,11 +314,15 @@ void MainTab::updateTextLabels(bool showUnits)
 
 void MainTab::enableEdition(EditMode newEditMode)
 {
+	const bool isTripEdit = MainWindow::instance() &&
+		MainWindow::instance()->dive_list()->selectedTrips().count() == 1;
+
 	if (((newEditMode == DIVE || newEditMode == NONE) && current_dive == NULL) || editMode != NONE)
 		return;
 	modified = false;
 	copyPaste = false;
 	if ((newEditMode == DIVE || newEditMode == NONE) &&
+	    !isTripEdit &&
 	    current_dive->dc.model &&
 	    strcmp(current_dive->dc.model, "manually added dive") == 0) {
 		// editCurrentDive will call enableEdition with newEditMode == MANUALLY_ADDED_DIVE
@@ -303,17 +337,14 @@ void MainTab::enableEdition(EditMode newEditMode)
 		MainWindow::instance()->editCurrentDive();
 		return;
 	}
+
+	ui.editDiveSiteButton->setEnabled(false);
 	MainWindow::instance()->dive_list()->setEnabled(false);
 	MainWindow::instance()->setEnabledToolbar(false);
 
-	// only setup the globe for editing if we are editing exactly one existing dive
-	if (amount_selected == 1 && newEditMode != ADD)
-		MainWindow::instance()->globe()->prepareForGetDiveCoordinates();
-
-	if (MainWindow::instance() && MainWindow::instance()->dive_list()->selectedTrips().count() == 1) {
+	if (isTripEdit) {
 		// we are editing trip location and notes
 		displayMessage(tr("This trip is being edited."));
-		memset(&displayedTrip, 0, sizeof(displayedTrip));
 		currentTrip = current_dive->divetrip;
 		ui.dateEdit->setEnabled(false);
 		editMode = TRIP;
@@ -385,12 +416,29 @@ bool MainTab::isEditing()
 	return editMode != NONE;
 }
 
+void MainTab::showLocation()
+{
+	if (get_dive_site_by_uuid(displayed_dive.dive_site_uuid))
+		ui.location->setCurrentDiveSiteUuid(displayed_dive.dive_site_uuid);
+	else
+		ui.location->clear();
+}
+
+// Seems wrong, since we can also call updateDiveInfo(), but since the updateDiveInfo
+// has a parameter on it's definition it didn't worked on the signal slot connection.
+void MainTab::refreshDiveInfo()
+{
+	updateDiveInfo();
+}
+
 void MainTab::updateDiveInfo(bool clear)
 {
+	ui.location->refreshDiveSiteCache();
+	EditMode rememberEM = editMode;
 	// don't execute this while adding / planning a dive
 	if (editMode == ADD || editMode == MANUALLY_ADDED_DIVE || MainWindow::instance()->graphics()->isPlanner())
 		return;
-	if (!isEnabled() && !clear)
+	if (!isEnabled() && !clear )
 		setEnabled(true);
 	if (isEnabled() && clear)
 		setEnabled(false);
@@ -418,18 +466,35 @@ void MainTab::updateDiveInfo(bool clear)
 		else
 			ui.notes->setPlainText(tmp);
 	}
-
 	UPDATE_TEXT(displayed_dive, notes);
-	UPDATE_TEXT(displayed_dive, location);
 	UPDATE_TEXT(displayed_dive, suit);
 	UPDATE_TEXT(displayed_dive, divemaster);
 	UPDATE_TEXT(displayed_dive, buddy);
 	UPDATE_TEMP(displayed_dive, airtemp);
 	UPDATE_TEMP(displayed_dive, watertemp);
-	ui.DiveType->setCurrentIndex(displayed_dive.dc.divemode);
+	ui.DiveType->setCurrentIndex(get_dive_dc(&displayed_dive, dc_number)->divemode);
 
 	if (!clear) {
-		updateGpsCoordinates();
+		struct dive_site *ds = NULL;
+		// if we are showing a dive and editing it, let's refer to the displayed_dive_site as that
+		// already may contain changes, otherwise start with the dive site referred to by the displayed
+		// dive
+		if (rememberEM == DIVE) {
+			ds = &displayed_dive_site;
+		} else {
+			ds = get_dive_site_by_uuid(displayed_dive.dive_site_uuid);
+			if (ds)
+				copy_dive_site(ds, &displayed_dive_site);
+		}
+
+		if (ds) {
+			ui.location->setCurrentDiveSiteUuid(ds->uuid);
+			ui.locationTags->setText(constructLocationTags(ds->uuid));
+		} else {
+			ui.location->clear();
+			clear_dive_site(&displayed_dive_site);
+		}
+
 		// Subsurface always uses "local time" as in "whatever was the local time at the location"
 		// so all time stamps have no time zone information and are in UTC
 		QDateTime localTime = QDateTime::fromTime_t(displayed_dive.when - gettimezoneoffset(displayed_dive.when));
@@ -440,8 +505,6 @@ void MainTab::updateDiveInfo(bool clear)
 			setTabText(0, tr("Trip notes"));
 			currentTrip = *MainWindow::instance()->dive_list()->selectedTrips().begin();
 			// only use trip relevant fields
-			ui.coordinates->setVisible(false);
-			ui.CoordinatedLabel->setVisible(false);
 			ui.divemaster->setVisible(false);
 			ui.DivemasterLabel->setVisible(false);
 			ui.buddy->setVisible(false);
@@ -460,19 +523,26 @@ void MainTab::updateDiveInfo(bool clear)
 			ui.TypeLabel->setVisible(false);
 			ui.waterTempLabel->setVisible(false);
 			ui.watertemp->setVisible(false);
+			ui.diveTripLocation->show();
+			ui.location->hide();
+			ui.editDiveSiteButton->hide();
 			// rename the remaining fields and fill data from selected trip
 			ui.LocationLabel->setText(tr("Trip location"));
-			ui.location->setText(currentTrip->location);
+			ui.diveTripLocation->setText(currentTrip->location);
+			ui.locationTags->clear();
+			//TODO: Fix this.
+			//ui.location->setText(currentTrip->location);
 			ui.NotesLabel->setText(tr("Trip notes"));
 			ui.notes->setText(currentTrip->notes);
 			clearEquipment();
 			ui.equipmentTab->setEnabled(false);
 		} else {
-			setTabText(0, tr("Dive notes"));
+			setTabText(0, tr("Notes"));
 			currentTrip = NULL;
 			// make all the fields visible writeable
-			ui.coordinates->setVisible(true);
-			ui.CoordinatedLabel->setVisible(true);
+			ui.diveTripLocation->hide();
+			ui.location->show();
+			ui.editDiveSiteButton->show();
 			ui.divemaster->setVisible(true);
 			ui.buddy->setVisible(true);
 			ui.suit->setVisible(true);
@@ -511,6 +581,7 @@ void MainTab::updateDiveInfo(bool clear)
 		ui.waterTemperatureText->setText(get_temperature_string(displayed_dive.watertemp, true));
 		ui.airTemperatureText->setText(get_temperature_string(displayed_dive.airtemp, true));
 		ui.DiveType->setCurrentIndex(get_dive_dc(&displayed_dive, dc_number)->divemode);
+
 		volume_t gases[MAX_CYLINDERS] = {};
 		get_gas_used(&displayed_dive, gases);
 		QString volumes;
@@ -635,21 +706,40 @@ void MainTab::updateDiveInfo(bool clear)
 				gasUsedString.append(QString("O2: %2\n").arg(get_volume_string(o2_tot, true)));
 		}
 		ui.gasConsumption->setText(gasUsedString);
+		if(ui.locationTags->text().isEmpty())
+			ui.locationTags->hide();
+		else
+			ui.locationTags->show();
+		/* unset the special value text for date and time, just in case someone dove at midnight */
+		ui.dateEdit->setSpecialValueText(QString(""));
+		ui.timeEdit->setSpecialValueText(QString(""));
+
 	} else {
 		/* clear the fields */
 		clearInfo();
 		clearStats();
 		clearEquipment();
 		ui.rating->setCurrentStars(0);
-		ui.coordinates->clear();
 		ui.visibility->setCurrentStars(0);
+		ui.location->clear();
+		/* set date and time to minimums which triggers showing the special value text */
+		ui.dateEdit->setSpecialValueText(QString("-"));
+		ui.dateEdit->setMinimumDate(QDate(1, 1, 1));
+		ui.dateEdit->setDate(QDate(1, 1, 1));
+		ui.timeEdit->setSpecialValueText(QString("-"));
+		ui.timeEdit->setMinimumTime(QTime(0, 0, 0, 0));
+		ui.timeEdit->setTime(QTime(0, 0, 0, 0));
 	}
-	editMode = NONE;
+	editMode = rememberEM;
 	ui.cylinders->view()->hideColumn(CylindersModel::DEPTH);
 	if (get_dive_dc(&displayed_dive, dc_number)->divemode == CCR)
 		ui.cylinders->view()->showColumn(CylindersModel::USE);
 	else
 		ui.cylinders->view()->hideColumn(CylindersModel::USE);
+
+	if (verbose)
+		qDebug() << "Set the current dive site:" << displayed_dive.dive_site_uuid;
+	emit diveSiteChanged(get_dive_site_by_uuid(displayed_dive.dive_site_uuid));
 }
 
 void MainTab::addCylinder_clicked()
@@ -670,9 +760,9 @@ void MainTab::reload()
 {
 	suitModel.updateModel();
 	buddyModel.updateModel();
-	locationModel.updateModel();
 	diveMasterModel.updateModel();
 	tagModel.updateModel();
+	LocationInformationModel::instance()->update();
 }
 
 // tricky little macro to edit all the selected dives
@@ -700,30 +790,107 @@ void MainTab::reload()
 		mydive->what = copy_string(displayed_dive.what); \
 	}
 
+MainTab::EditMode MainTab::getEditMode() const
+{
+	return editMode;
+}
+
 #define EDIT_VALUE(what)                             \
 	if (mydive->what == cd->what || copyPaste) { \
 		mydive->what = displayed_dive.what;  \
 	}
 
+void MainTab::refreshDisplayedDiveSite()
+{
+	if (displayed_dive_site.uuid) {
+		copy_dive_site(get_dive_site_by_uuid(displayed_dive_site.uuid), &displayed_dive_site);
+		ui.location->setCurrentDiveSiteUuid(displayed_dive_site.uuid);
+	}
+}
+
+// when this is called we already have updated the current_dive and know that it exists
+// there is no point in calling this function if there is no current dive
+uint32_t MainTab::updateDiveSite(uint32_t pickedUuid, int divenr)
+{
+	struct dive *cd = get_dive(divenr);
+	if (!cd)
+		return 0;
+
+	if (ui.location->text().isEmpty())
+		return 0;
+
+	if (pickedUuid == 0)
+		return 0;
+
+	const uint32_t origUuid = cd->dive_site_uuid;
+	struct dive_site *origDs = get_dive_site_by_uuid(origUuid);
+	struct dive_site *newDs = NULL;
+	bool createdNewDive = false;
+
+	if (pickedUuid == origUuid)
+		return origUuid;
+
+	if (pickedUuid == RECENTLY_ADDED_DIVESITE) {
+		pickedUuid = create_dive_site(ui.location->text().isEmpty() ? qPrintable(tr("New dive site")) : qPrintable(ui.location->text()), displayed_dive.when);
+		createdNewDive = true;
+	}
+
+	newDs = get_dive_site_by_uuid(pickedUuid);
+
+	// Copy everything from the displayed_dive_site, so we have the latitude, longitude, notes, etc.
+	// The user *might* be using wrongly the 'choose dive site' just to edit the name of it, sigh.
+	if (origDs) {
+		if(createdNewDive) {
+			copy_dive_site(origDs, newDs);
+			free(newDs->name);
+			newDs->name = copy_string(qPrintable(ui.location->text().constData()));
+			newDs->uuid = pickedUuid;
+			qDebug() << "Creating and copying dive site";
+		} else if (newDs->latitude.udeg == 0 && newDs->longitude.udeg == 0) {
+			newDs->latitude.udeg = origDs->latitude.udeg;
+			newDs->longitude.udeg = origDs->longitude.udeg;
+			qDebug() << "Copying GPS information";
+		}
+	}
+
+	if (origDs && pickedUuid != origDs->uuid && same_string(origDs->notes, "SubsurfaceWebservice")) {
+		if (!is_dive_site_used(origDs->uuid, false)) {
+			if (verbose)
+				qDebug() << "delete the autogenerated dive site" << origDs->name;
+			delete_dive_site(origDs->uuid);
+		}
+	}
+
+	cd->dive_site_uuid = pickedUuid;
+	qDebug() << "Setting the dive site id on the dive:" << pickedUuid;
+	return pickedUuid;
+}
+
 void MainTab::acceptChanges()
 {
 	int i, addedId = -1;
 	struct dive *d;
 	bool do_replot = false;
 
+	if(ui.location->hasFocus()) {
+		this->setFocus();
+	}
+
 	acceptingEdit = true;
 	tabBar()->setTabIcon(0, QIcon()); // Notes
 	tabBar()->setTabIcon(1, QIcon()); // Equipment
 	ui.dateEdit->setEnabled(true);
 	hideMessage();
 	ui.equipmentTab->setEnabled(true);
-	on_location_editingFinished(); // complete coordinates *before* saving
 	if (editMode == ADD) {
 		// We need to add the dive we just created to the dive list and select it.
 		// Easy, right?
 		struct dive *added_dive = clone_dive(&displayed_dive);
 		record_dive(added_dive);
 		addedId = added_dive->id;
+		// make sure that the dive site is handled as well
+		updateDiveSite(ui.location->currDiveSiteUuid(), get_idx_by_uniq_id(added_dive->id));
+
 		// unselect everything as far as the UI is concerned and select the new
 		// dive - we'll have to undo/redo this later after we resort the dive_table
 		// but we need the dive selected for the middle part of this function - this
@@ -748,29 +915,29 @@ void MainTab::acceptChanges()
 			// preserve any changes to the profile
 			free(current_dive->dc.sample);
 			copy_samples(&displayed_dive.dc, &current_dive->dc);
+			addedId = displayed_dive.id;
 		}
 		struct dive *cd = current_dive;
-		//Reset coordinates field, in case it contains garbage.
-		updateGpsCoordinates();
+		struct divecomputer *displayed_dc = get_dive_dc(&displayed_dive, dc_number);
 		// now check if something has changed and if yes, edit the selected dives that
 		// were identical with the master dive shown (and mark the divelist as changed)
-		if (!same_string(displayed_dive.buddy, cd->buddy))
-			MODIFY_SELECTED_DIVES(EDIT_TEXT(buddy));
 		if (!same_string(displayed_dive.suit, cd->suit))
 			MODIFY_SELECTED_DIVES(EDIT_TEXT(suit));
 		if (!same_string(displayed_dive.notes, cd->notes))
 			MODIFY_SELECTED_DIVES(EDIT_TEXT(notes));
-		if (!same_string(displayed_dive.divemaster, cd->divemaster))
-			MODIFY_SELECTED_DIVES(EDIT_TEXT(divemaster));
 		if (displayed_dive.rating != cd->rating)
 			MODIFY_SELECTED_DIVES(EDIT_VALUE(rating));
 		if (displayed_dive.visibility != cd->visibility)
 			MODIFY_SELECTED_DIVES(EDIT_VALUE(visibility));
 		if (displayed_dive.airtemp.mkelvin != cd->airtemp.mkelvin)
 			MODIFY_SELECTED_DIVES(EDIT_VALUE(airtemp.mkelvin));
-		if (displayed_dive.dc.divemode != cd->dc.divemode) {
-			MODIFY_SELECTED_DIVES(EDIT_VALUE(dc.divemode));
-			MODIFY_SELECTED_DIVES(update_setpoint_events(&mydive->dc));
+		if (displayed_dc->divemode != current_dc->divemode) {
+			MODIFY_SELECTED_DIVES(
+				if (get_dive_dc(mydive, dc_number)->divemode == current_dc->divemode || copyPaste) {
+					get_dive_dc(mydive, dc_number)->divemode = displayed_dc->divemode;
+				}
+			);
+			MODIFY_SELECTED_DIVES(update_setpoint_events(get_dive_dc(mydive, dc_number)));
 			do_replot = true;
 		}
 		if (displayed_dive.watertemp.mkelvin != cd->watertemp.mkelvin)
@@ -779,18 +946,13 @@ void MainTab::acceptChanges()
 			time_t offset = cd->when - displayed_dive.when;
 			MODIFY_SELECTED_DIVES(mydive->when -= offset;);
 		}
-		if (displayed_dive.latitude.udeg != cd->latitude.udeg ||
-		    displayed_dive.longitude.udeg != cd->longitude.udeg)
-			MODIFY_SELECTED_DIVES(
-				if (copyPaste ||
-				    (same_string(mydive->location, cd->location) &&
-				     mydive->latitude.udeg == cd->latitude.udeg &&
-				     mydive->longitude.udeg == cd->longitude.udeg))
-					gpsHasChanged(mydive, cd, ui.coordinates->text(), 0);
-			);
-		if (!same_string(displayed_dive.location, cd->location))
-			MODIFY_SELECTED_DIVES(EDIT_TEXT(location));
 
+		if (displayed_dive.dive_site_uuid != cd->dive_site_uuid)
+			MODIFY_SELECTED_DIVES(EDIT_VALUE(dive_site_uuid));
+
+		// three text fields are somewhat special and are represented as tags
+		// in the UI - they need somewhat smarter handling
+		saveTaggedStrings();
 		saveTags();
 
 		if (editMode != ADD && cylindersModel->changed) {
@@ -821,6 +983,16 @@ void MainTab::acceptChanges()
 				cd->cylinder[i] = displayed_dive.cylinder[i];
 				cd->cylinder[i].type.description = copy_string(displayed_dive.cylinder[i].type.description);
 			}
+			/* if cylinders changed we may have changed gas change events
+			 * - so far this is ONLY supported for a single selected dive */
+			struct divecomputer *tdc = &current_dive->dc;
+			struct divecomputer *sdc = &displayed_dive.dc;
+			while(tdc && sdc) {
+				free_events(tdc->events);
+				copy_events(sdc, tdc);
+				tdc = tdc->next;
+				sdc = sdc->next;
+			}
 			do_replot = true;
 		}
 
@@ -839,6 +1011,32 @@ void MainTab::acceptChanges()
 				cd->weightsystem[i].description = copy_string(displayed_dive.weightsystem[i].description);
 			}
 		}
+
+		// update the dive site for the selected dives that had the same dive site as the current dive
+		uint32_t oldUuid = cd->dive_site_uuid;
+		uint32_t newUuid = 0;
+		MODIFY_SELECTED_DIVES(
+			if (mydive->dive_site_uuid == current_dive->dive_site_uuid) {
+				newUuid = updateDiveSite(newUuid == 0 ? ui.location->currDiveSiteUuid() : newUuid,  get_idx_by_uniq_id(mydive->id));
+			}
+		);
+		if (!is_dive_site_used(oldUuid, false)) {
+			if (verbose) {
+				struct dive_site *ds = get_dive_site_by_uuid(oldUuid);
+				qDebug() << "delete now unused dive site" << ((ds && ds->name) ? ds->name : "without name");
+			}
+			delete_dive_site(oldUuid);
+			GlobeGPS::instance()->reload();
+		}
+		// the code above can change the correct uuid for the displayed dive site - and the
+		// code below triggers an update of the display without re-initializing displayed_dive
+		// so let's make sure here that our data is consistent now that we have handled the
+		// dive sites
+		displayed_dive.dive_site_uuid = current_dive->dive_site_uuid;
+		struct dive_site *ds = get_dive_site_by_uuid(displayed_dive.dive_site_uuid);
+		if (ds)
+			copy_dive_site(ds, &displayed_dive_site);
+
 		// each dive that was selected might have had the temperatures in its active divecomputer changed
 		// so re-populate the temperatures - easiest way to do this is by calling fixup_dive
 		for_each_dive (i, d) {
@@ -846,11 +1044,14 @@ void MainTab::acceptChanges()
 				fixup_dive(d);
 		}
 	}
-	if (current_dive->divetrip) {
+	if (editMode != TRIP && current_dive->divetrip) {
 		current_dive->divetrip->when = current_dive->when;
 		find_new_trip_start_time(current_dive->divetrip);
 	}
 	if (editMode == ADD || editMode == MANUALLY_ADDED_DIVE) {
+		// we just added or edited the dive, let fixup_dive() make
+		// sure we get the max depth right
+		current_dive->maxdepth.mm = current_dc->maxdepth.mm = 0;
 		fixup_dive(current_dive);
 		set_dive_nr_for_current_dive();
 		MainWindow::instance()->showProfile();
@@ -887,6 +1088,7 @@ void MainTab::acceptChanges()
 	weightModel->changed = false;
 	MainWindow::instance()->setEnabledToolbar(true);
 	acceptingEdit = false;
+	ui.editDiveSiteButton->setEnabled(true);
 }
 
 void MainTab::resetPallete()
@@ -895,7 +1097,6 @@ void MainTab::resetPallete()
 	ui.buddy->setPalette(p);
 	ui.notes->setPalette(p);
 	ui.location->setPalette(p);
-	ui.coordinates->setPalette(p);
 	ui.divemaster->setPalette(p);
 	ui.suit->setPalette(p);
 	ui.airtemp->setPalette(p);
@@ -904,6 +1105,7 @@ void MainTab::resetPallete()
 	ui.dateEdit->setPalette(p);
 	ui.timeEdit->setPalette(p);
 	ui.tagWidget->setPalette(p);
+	ui.diveTripLocation->setPalette(p);
 }
 
 #define EDIT_TEXT2(what, text)         \
@@ -949,8 +1151,9 @@ void MainTab::rejectChanges()
 	DivePictureModel::instance()->updateDivePictures();
 	// the user could have edited the location and then canceled the edit
 	// let's get the correct location back in view
-	MainWindow::instance()->globe()->centerOnCurrentDive();
-	MainWindow::instance()->globe()->reload();
+#ifndef NO_MARBLE
+	GlobeGPS::instance()->centerOnDiveSite(get_dive_site_by_uuid(displayed_dive.dive_site_uuid));
+#endif
 	// show the profile and dive info
 	MainWindow::instance()->graphics()->replot();
 	MainWindow::instance()->setEnabledToolbar(true);
@@ -959,6 +1162,7 @@ void MainTab::rejectChanges()
 	cylindersModel->updateDive();
 	weightModel->updateDive();
 	extraDataModel->updateDive();
+	ui.editDiveSiteButton->setEnabled(true);
 }
 #undef EDIT_TEXT2
 
@@ -970,10 +1174,7 @@ void MainTab::markChangedWidget(QWidget *w)
 	qApp->palette().color(QPalette::Text).getHslF(&h, &s, &l, &a);
 	p.setBrush(QPalette::Base, (l <= 0.3) ? QColor(Qt::yellow).lighter() : (l <= 0.6) ? QColor(Qt::yellow).light() : /* else */ QColor(Qt::yellow).darker(300));
 	w->setPalette(p);
-	if (!modified) {
-		modified = true;
-		enableEdition();
-	}
+	modified = true;
 }
 
 void MainTab::on_buddy_textChanged()
@@ -1023,8 +1224,9 @@ void MainTab::divetype_Changed(int index)
 {
 	if (editMode == IGNORE)
 		return;
-	displayed_dive.dc.divemode = (enum dive_comp_type) index;
-	update_setpoint_events(&displayed_dive.dc);
+	struct divecomputer *displayed_dc = get_dive_dc(&displayed_dive, dc_number);
+	displayed_dc->divemode = (enum dive_comp_type) index;
+	update_setpoint_events(displayed_dc);
 	markChangedWidget(ui.DiveType);
 	MainWindow::instance()->graphics()->recalcCeiling();
 }
@@ -1135,6 +1337,75 @@ void MainTab::saveTags()
 		taglist_free(mydive->tag_list);
 		mydive->tag_list = new_tag_list;
 	);
+	taglist_free(added_list);
+	taglist_free(removed_list);
+}
+
+// buddy and divemaster are represented in the UI just like the tags, but the internal
+// representation is just a string (with commas as delimiters). So we need to do the same
+// thing we did for tags, just differently
+void MainTab::saveTaggedStrings()
+{
+	QStringList addedList, removedList;
+	struct dive *cd = current_dive;
+
+	diffTaggedStrings(cd->buddy, displayed_dive.buddy, addedList, removedList);
+	MODIFY_SELECTED_DIVES(
+		QStringList oldList = QString(mydive->buddy).split(QRegExp("\\s*,\\s*"), QString::SkipEmptyParts);
+		QString newString;
+		QString comma;
+		Q_FOREACH (const QString tag, oldList) {
+			if (!removedList.contains(tag, Qt::CaseInsensitive)) {
+				newString += comma + tag;
+				comma = ", ";
+			}
+		}
+		Q_FOREACH (const QString tag, addedList) {
+			if (!oldList.contains(tag, Qt::CaseInsensitive)) {
+				newString += comma + tag;
+				comma = ", ";
+			}
+		}
+		free(mydive->buddy);
+		mydive->buddy = copy_string(qPrintable(newString));
+	);
+	addedList.clear();
+	removedList.clear();
+	diffTaggedStrings(cd->divemaster, displayed_dive.divemaster, addedList, removedList);
+	MODIFY_SELECTED_DIVES(
+		QStringList oldList = QString(mydive->divemaster).split(QRegExp("\\s*,\\s*"), QString::SkipEmptyParts);
+		QString newString;
+		QString comma;
+		Q_FOREACH (const QString tag, oldList) {
+			if (!removedList.contains(tag, Qt::CaseInsensitive)) {
+				newString += comma + tag;
+				comma = ", ";
+			}
+		}
+		Q_FOREACH (const QString tag, addedList) {
+			if (!oldList.contains(tag, Qt::CaseInsensitive)) {
+				newString += comma + tag;
+				comma = ", ";
+			}
+		}
+		free(mydive->divemaster);
+		mydive->divemaster = copy_string(qPrintable(newString));
+	);
+}
+
+void MainTab::diffTaggedStrings(QString currentString, QString displayedString, QStringList &addedList, QStringList &removedList)
+{
+	QStringList displayedList, currentList;
+	currentList = currentString.split(',', QString::SkipEmptyParts);
+	displayedList = displayedString.split(',', QString::SkipEmptyParts);
+	Q_FOREACH ( const QString tag, currentList) {
+		if (!displayedList.contains(tag, Qt::CaseInsensitive))
+			removedList << tag.trimmed();
+	}
+	Q_FOREACH (const QString tag, displayedList) {
+		if (!currentList.contains(tag, Qt::CaseInsensitive))
+			addedList << tag.trimmed();
+	}
 }
 
 void MainTab::on_tagWidget_textChanged()
@@ -1151,41 +1422,47 @@ void MainTab::on_tagWidget_textChanged()
 	markChangedWidget(ui.tagWidget);
 }
 
-void MainTab::on_location_textChanged(const QString &text)
+void MainTab::on_location_textChanged()
+{
+	if (editMode == IGNORE)
+		return;
+
+	// we don't want to act on the edit until editing is finished,
+	// but we want to mark the field so it's obvious it is being edited
+	QString currentLocation;
+	struct dive_site *ds = get_dive_site_by_uuid(displayed_dive.dive_site_uuid);
+	if (ds)
+		currentLocation = ds->name;
+	if (ui.location->text() != currentLocation)
+		markChangedWidget(ui.location);
+}
+
+void MainTab::on_location_diveSiteSelected()
 {
 	if (editMode == IGNORE || acceptingEdit == true)
 		return;
-	if (currentTrip) {
-		free(displayedTrip.location);
-		displayedTrip.location = strdup(ui.location->text().toUtf8().data());
+
+	if (ui.location->text().isEmpty()) {
+		displayed_dive.dive_site_uuid = 0;
+		markChangedWidget(ui.location);
+		emit diveSiteChanged(0);
+		return;
 	} else {
-		free(displayed_dive.location);
-		displayed_dive.location = strdup(ui.location->text().toUtf8().data());
+		if (ui.location->currDiveSiteUuid() != displayed_dive.dive_site_uuid) {
+			markChangedWidget(ui.location);
+		} else {
+			QPalette p;
+			ui.location->setPalette(p);
+		}
 	}
-	markChangedWidget(ui.location);
 }
 
-// If we have GPS data for the location entered, add it.
-void MainTab::on_location_editingFinished()
+void MainTab::on_diveTripLocation_textEdited(const QString& text)
 {
-	// if we have a location and no GPS data, look up the GPS data;
-	// but if the GPS data was intentionally cleared then don't
-	if (!currentTrip &&
-	    !same_string(displayed_dive.location, "") &&
-	    ui.coordinates->text().trimmed().isEmpty() &&
-	    !(editMode == DIVE && dive_has_gps_location(current_dive))) {
-		struct dive *dive;
-		int i = 0;
-		for_each_dive (i, dive) {
-			if (same_string(displayed_dive.location, dive->location) &&
-			    (dive->latitude.udeg || dive->longitude.udeg)) {
-				displayed_dive.latitude = dive->latitude;
-				displayed_dive.longitude = dive->longitude;
-				MainWindow::instance()->globe()->reload();
-				updateGpsCoordinates();
-				break;
-			}
-		}
+	if (currentTrip) {
+		free(displayedTrip.location);
+		displayedTrip.location = strdup(qPrintable(text));
+		markChangedWidget(ui.diveTripLocation);
 	}
 }
 
@@ -1219,23 +1496,6 @@ void MainTab::on_notes_textChanged()
 	markChangedWidget(ui.notes);
 }
 
-void MainTab::on_coordinates_textChanged(const QString &text)
-{
-	if (editMode == IGNORE || acceptingEdit == true)
-		return;
-	bool gpsChanged = false;
-	bool parsed = false;
-	QPalette p;
-	ui.coordinates->setPalette(p); // reset palette
-	gpsChanged = gpsHasChanged(&displayed_dive, &displayed_dive, text, &parsed);
-	if (gpsChanged)
-		markChangedWidget(ui.coordinates); // marks things yellow
-	if (!parsed) {
-		p.setBrush(QPalette::Base, QColor(Qt::red).lighter());
-		ui.coordinates->setPalette(p); // marks things red
-	}
-}
-
 void MainTab::on_rating_valueChanged(int value)
 {
 	if (acceptingEdit == true)
@@ -1286,22 +1546,6 @@ void MainTab::editWeightWidget(const QModelIndex &index)
 		ui.weights->edit(index);
 }
 
-void MainTab::updateCoordinatesText(qreal lat, qreal lon)
-{
-	int ulat = rint(lat * 1000000);
-	int ulon = rint(lon * 1000000);
-	ui.coordinates->setText(printGPSCoords(ulat, ulon));
-}
-
-void MainTab::updateGpsCoordinates()
-{
-	if (editMode == NONE)
-		enableEdition();
-
-	ui.coordinates->setText(printGPSCoords(displayed_dive.latitude.udeg, displayed_dive.longitude.udeg));
-	ui.coordinates->setModified(displayed_dive.latitude.udeg || displayed_dive.longitude.udeg);
-}
-
 void MainTab::escDetected()
 {
 	if (editMode != NONE)
@@ -1332,7 +1576,6 @@ void MainTab::showAndTriggerEditSelective(struct dive_components what)
 	// take the data in our copyPasteDive and apply it to selected dives
 	enableEdition();
 	copyPaste = true;
-	SHOW_SELECTIVE(location);
 	SHOW_SELECTIVE(buddy);
 	SHOW_SELECTIVE(divemaster);
 	SHOW_SELECTIVE(suit);
@@ -1347,8 +1590,8 @@ void MainTab::showAndTriggerEditSelective(struct dive_components what)
 		ui.rating->setCurrentStars(displayed_dive.rating);
 	if (what.visibility)
 		ui.visibility->setCurrentStars(displayed_dive.visibility);
-	if (what.gps)
-		updateGpsCoordinates();
+	if (what.divesite)
+		ui.location->setCurrentDiveSiteUuid(displayed_dive.dive_site_uuid);
 	if (what.tags) {
 		char buf[1024];
 		taglist_get_tagstring(displayed_dive.tag_list, buf, 1024);
