@@ -62,7 +62,7 @@ void DiveEventItem::setupPixmap()
 
 #define EVENT_PIXMAP(PIX) QPixmap(QString(PIX)).scaled(sz_pix, sz_pix, Qt::KeepAspectRatio, Qt::SmoothTransformation)
 #define EVENT_PIXMAP_BIGGER(PIX) QPixmap(QString(PIX)).scaled(sz_bigger, sz_bigger, Qt::KeepAspectRatio, Qt::SmoothTransformation)
-	if (!internalEvent->name) {
+	if (same_string(internalEvent->name, "")) {
 		setPixmap(EVENT_PIXMAP(":warning"));
 	} else if (internalEvent->type == SAMPLE_EVENT_BOOKMARK) {
 		setPixmap(EVENT_PIXMAP(":flag"));
@@ -135,7 +135,9 @@ bool DiveEventItem::shouldBeHidden()
 	 * Don't bother showing those
 	 */
 	struct sample *first_sample = &get_dive_dc(&displayed_dive, dc_number)->sample[0];
-	if (!strcmp(event->name, "gaschange") && first_sample && event->time.seconds == first_sample->time.seconds)
+	if (!strcmp(event->name, "gaschange") &&
+	    (event->time.seconds == 0 ||
+	     (first_sample && event->time.seconds == first_sample->time.seconds)))
 		return true;
 
 	for (int i = 0; i < evn_used; i++) {
