@@ -5,12 +5,17 @@
 #include <QFileDialog>
 #include <QShortcut>
 #include <QCalendarWidget>
+#include <QKeyEvent>
+#include <QAction>
 
 #include "file.h"
 #include "mainwindow.h"
 #include "helpers.h"
 #include "libdivecomputer/parser.h"
-
+#include "divelistview.h"
+#include "display.h"
+#include "profile/profilewidget2.h"
+#include "undocommands.h"
 
 class MinMaxAvgWidgetPrivate {
 public:
@@ -137,8 +142,24 @@ void RenumberDialog::renumberOnlySelected(bool selected)
 
 void RenumberDialog::buttonClicked(QAbstractButton *button)
 {
-	if (ui.buttonBox->buttonRole(button) == QDialogButtonBox::AcceptRole)
-		renumber_dives(ui.spinBox->value(), selectedOnly);
+	if (ui.buttonBox->buttonRole(button) == QDialogButtonBox::AcceptRole) {
+		MainWindow::instance()->dive_list()->rememberSelection();
+		// we remember a map from dive uuid to a pair of old number / new number
+		QMap<int,QPair<int, int> > renumberedDives;
+		int i;
+		int newNr = ui.spinBox->value();
+		struct dive *dive = NULL;
+		for_each_dive (i, dive) {
+			if (!selectedOnly || dive->selected)
+				renumberedDives.insert(dive->id, QPair<int,int>(dive->number, newNr++));
+		}
+		UndoRenumberDives *undoCommand = new UndoRenumberDives(renumberedDives);
+		MainWindow::instance()->undoStack->push(undoCommand);
+
+		MainWindow::instance()->dive_list()->fixMessyQtModelBehaviour();
+		mark_divelist_changed(true);
+		MainWindow::instance()->dive_list()->restoreSelection();
+	}
 }
 
 RenumberDialog::RenumberDialog(QWidget *parent) : QDialog(parent), selectedOnly(false)
@@ -165,14 +186,15 @@ void SetpointDialog::setpointData(struct divecomputer *divecomputer, int second)
 
 void SetpointDialog::buttonClicked(QAbstractButton *button)
 {
-	if (ui.buttonBox->buttonRole(button) == QDialogButtonBox::AcceptRole)
+	if (ui.buttonBox->buttonRole(button) == QDialogButtonBox::AcceptRole && dc)
 		add_event(dc, time, SAMPLE_EVENT_PO2, 0, (int)(1000.0 * ui.spinbox->value()), "SP change");
 	mark_divelist_changed(true);
 	MainWindow::instance()->graphics()->replot();
-
 }
 
-SetpointDialog::SetpointDialog(QWidget *parent) : QDialog(parent)
+SetpointDialog::SetpointDialog(QWidget *parent) :
+	QDialog(parent),
+	dc(0)
 {
 	ui.setupUi(this);
 	connect(ui.buttonBox, SIGNAL(clicked(QAbstractButton *)), this, SLOT(buttonClicked(QAbstractButton *)));
@@ -198,7 +220,16 @@ void ShiftTimesDialog::buttonClicked(QAbstractButton *button)
 			amount *= -1;
 		if (amount != 0) {
 			// DANGER, DANGER - this could get our dive_table unsorted...
-			shift_times(amount);
+			int i;
+			struct dive *dive;
+			QList<int> affectedDives;
+			for_each_dive (i, dive) {
+				if (!dive->selected)
+					continue;
+
+				affectedDives.append(dive->id);
+			}
+			MainWindow::instance()->undoStack->push(new UndoShiftTime(affectedDives, amount));
 			sort_table(&dive_table);
 			mark_divelist_changed(true);
 			MainWindow::instance()->dive_list()->rememberSelection();
@@ -227,7 +258,9 @@ void ShiftTimesDialog::changeTime()
 	ui.shiftedTime->setText(get_dive_date_string(amount + when));
 }
 
-ShiftTimesDialog::ShiftTimesDialog(QWidget *parent) : QDialog(parent)
+ShiftTimesDialog::ShiftTimesDialog(QWidget *parent) :
+	QDialog(parent),
+	when(0)
 {
 	ui.setupUi(this);
 	connect(ui.buttonBox, SIGNAL(clicked(QAbstractButton *)), this, SLOT(buttonClicked(QAbstractButton *)));
@@ -250,9 +283,6 @@ void ShiftImageTimesDialog::buttonClicked(QAbstractButton *button)
 
 void ShiftImageTimesDialog::syncCameraClicked()
 {
-	struct memblock mem;
-	EXIFInfo exiv;
-	int retval;
 	QPixmap picture;
 	QDateTime dcDateTime = QDateTime();
 	QStringList fileNames = QFileDialog::getOpenFileNames(this,
@@ -268,30 +298,43 @@ void ShiftImageTimesDialog::syncCameraClicked()
 
 	scene->addPixmap(picture.scaled(ui.DCImage->size()));
 	ui.DCImage->setScene(scene);
-	if (readfile(fileNames.at(0).toUtf8().data(), &mem) <= 0)
-		return;
-	retval = exiv.parseFrom((const unsigned char *)mem.buffer, (unsigned)mem.size);
-	free(mem.buffer);
-	if (retval != PARSE_EXIF_SUCCESS)
-		return;
-	dcImageEpoch = exiv.epoch();
-	dcDateTime.setTime_t(dcImageEpoch);
+
+	dcImageEpoch = picture_get_timestamp(fileNames.at(0).toUtf8().data());
+	dcDateTime.setTime_t(dcImageEpoch - gettimezoneoffset(displayed_dive.when));
 	ui.dcTime->setDateTime(dcDateTime);
 	connect(ui.dcTime, SIGNAL(dateTimeChanged(const QDateTime &)), this, SLOT(dcDateTimeChanged(const QDateTime &)));
 }
 
 void ShiftImageTimesDialog::dcDateTimeChanged(const QDateTime &newDateTime)
 {
+	QDateTime newtime(newDateTime);
 	if (!dcImageEpoch)
 		return;
-	setOffset(newDateTime.toTime_t() - dcImageEpoch);
+	newtime.setTimeSpec(Qt::UTC);
+	setOffset(newtime.toTime_t() + gettimezoneoffset(displayed_dive.when) - dcImageEpoch);
 }
 
-ShiftImageTimesDialog::ShiftImageTimesDialog(QWidget *parent) : QDialog(parent), m_amount(0)
+void ShiftImageTimesDialog::matchAllImagesToggled(bool state)
+{
+	matchAllImages = state;
+}
+
+bool ShiftImageTimesDialog::matchAll()
+{
+	return matchAllImages;
+}
+
+ShiftImageTimesDialog::ShiftImageTimesDialog(QWidget *parent, QStringList fileNames) :
+	QDialog(parent),
+	fileNames(fileNames),
+	m_amount(0),
+	matchAllImages(false)
 {
 	ui.setupUi(this);
 	connect(ui.buttonBox, SIGNAL(clicked(QAbstractButton *)), this, SLOT(buttonClicked(QAbstractButton *)));
 	connect(ui.syncCamera, SIGNAL(clicked()), this, SLOT(syncCameraClicked()));
+	connect(ui.timeEdit, SIGNAL(timeChanged(const QTime &)), this, SLOT(timeEditChanged(const QTime &)));
+	connect(ui.matchAllImages, SIGNAL(toggled(bool)), this, SLOT(matchAllImagesToggled(bool)));
 	dcImageEpoch = (time_t)0;
 }
 
@@ -311,6 +354,56 @@ void ShiftImageTimesDialog::setOffset(time_t offset)
 	ui.timeEdit->setTime(QTime(offset / 3600, (offset % 3600) / 60, offset % 60));
 }
 
+void ShiftImageTimesDialog::updateInvalid()
+{
+	timestamp_t timestamp;
+	QDateTime time;
+	bool allValid = true;
+	ui.warningLabel->hide();
+	ui.invalidLabel->hide();
+	time.setTime_t(displayed_dive.when - gettimezoneoffset(displayed_dive.when));
+	ui.invalidLabel->setText("Dive:" + time.toString() + "\n");
+
+	Q_FOREACH (const QString &fileName, fileNames) {
+		if (picture_check_valid(fileName.toUtf8().data(), m_amount))
+			continue;
+
+		// We've found invalid image
+		timestamp = picture_get_timestamp(fileName.toUtf8().data());
+		dcImageEpoch = timestamp;
+		time.setTime_t(timestamp + m_amount - gettimezoneoffset(displayed_dive.when));
+		ui.invalidLabel->setText(ui.invalidLabel->text() + fileName + " " + time.toString() + "\n");
+		allValid = false;
+	}
+
+	if (!allValid){
+		ui.warningLabel->show();
+		ui.invalidLabel->show();
+	}
+}
+
+void ShiftImageTimesDialog::timeEditChanged(const QTime &time)
+{
+	m_amount = time.hour() * 3600 + time.minute() * 60;
+	if (ui.backwards->isChecked())
+			m_amount *= -1;
+	updateInvalid();
+}
+
+URLDialog::URLDialog(QWidget *parent) : QDialog(parent)
+{
+	ui.setupUi(this);
+	QShortcut *close = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_W), this);
+	connect(close, SIGNAL(activated()), this, SLOT(close()));
+	QShortcut *quit = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_Q), this);
+	connect(quit, SIGNAL(activated()), parent, SLOT(close()));
+}
+
+QString URLDialog::url() const
+{
+	return ui.urlField->toPlainText();
+}
+
 bool isGnome3Session()
 {
 #if defined(QT_OS_WIW) || defined(QT_OS_MAC)
@@ -454,8 +547,7 @@ DiveComponentSelection::DiveComponentSelection(QWidget *parent, struct dive *tar
 {
 	ui.setupUi(this);
 	what = _what;
-	UI_FROM_COMPONENT(location);
-	UI_FROM_COMPONENT(gps);
+	UI_FROM_COMPONENT(divesite);
 	UI_FROM_COMPONENT(divemaster);
 	UI_FROM_COMPONENT(buddy);
 	UI_FROM_COMPONENT(rating);
@@ -475,8 +567,7 @@ DiveComponentSelection::DiveComponentSelection(QWidget *parent, struct dive *tar
 void DiveComponentSelection::buttonClicked(QAbstractButton *button)
 {
 	if (ui.buttonBox->buttonRole(button) == QDialogButtonBox::AcceptRole) {
-		COMPONENT_FROM_UI(location);
-		COMPONENT_FROM_UI(gps);
+		COMPONENT_FROM_UI(divesite);
 		COMPONENT_FROM_UI(divemaster);
 		COMPONENT_FROM_UI(buddy);
 		COMPONENT_FROM_UI(rating);
