@@ -1,8 +1,10 @@
 #include "statisticswidget.h"
-#include "models.h"
+#include "yearlystatisticsmodel.h"
 #include <QModelIndex>
 
-YearlyStatisticsWidget::YearlyStatisticsWidget(QWidget *parent): QGraphicsView(parent)
+YearlyStatisticsWidget::YearlyStatisticsWidget(QWidget *parent):
+	QGraphicsView(parent),
+	m_model(NULL)
 {
 }
 
