@@ -1,7 +1,13 @@
 #include "subsurfacewebservices.h"
+#include "helpers.h"
 #include "webservice.h"
 #include "mainwindow.h"
 #include "usersurvey.h"
+#include "divelist.h"
+#include "globe.h"
+#include "maintab.h"
+#include "display.h"
+#include "membuffer.h"
 #include <errno.h>
 
 #include <QDir>
@@ -17,16 +23,32 @@
 #include <unistd.h> // for dup(2)
 #endif
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 #include <QUrlQuery>
-#endif
 
 #ifndef PATH_MAX
 #define PATH_MAX 4096
 #endif
 
 struct dive_table gps_location_table;
-static bool merge_locations_into_dives(void);
+
+// we don't overwrite any existing GPS info in the dive
+// so get the dive site and if there is none or there is one without GPS fix, add it
+static void copy_gps_location(struct dive *from, struct dive *to)
+{
+	struct dive_site *ds = get_dive_site_for_dive(to);
+	if (!ds || !dive_site_has_gps_location(ds)) {
+		struct dive_site *gds = get_dive_site_for_dive(from);
+		if (!ds) {
+			// simply link to the one created for the fake dive
+			to->dive_site_uuid = gds->uuid;
+		} else {
+			ds->latitude = gds->latitude;
+			ds->longitude = gds->longitude;
+			if (same_string(ds->name, ""))
+				ds->name = copy_string(gds->name);
+		}
+	}
+}
 
 #define SAME_GROUP 6 * 3600 // six hours
 //TODO: C Code. static functions are not good if we plan to have a test for them.
@@ -39,14 +61,21 @@ static bool merge_locations_into_dives(void)
 
 	for_each_dive (i, dive) {
 		if (!dive_has_gps_location(dive)) {
-			for (j = tracer; (gpsfix = get_gps_location(j, &gps_location_table)) !=NULL; j++) {
-				if (dive_within_time_range (dive, gpsfix->when, SAME_GROUP)) {
+			for (j = tracer; (gpsfix = get_dive_from_table(j, &gps_location_table)) !=NULL; j++) {
+				if (time_during_dive_with_offset(dive, gpsfix->when, SAME_GROUP)) {
+					if (verbose)
+						qDebug() << "processing gpsfix @" << get_dive_date_string(gpsfix->when) <<
+							    "which is withing six hours of dive from" <<
+							    get_dive_date_string(dive->when) << "until" <<
+							    get_dive_date_string(dive->when + dive->duration.seconds);
 					/*
 					 * If position is fixed during dive. This is the good one.
 					 * Asign and mark position, and end gps_location loop
 					 */
-					if ((dive->when <= gpsfix->when && gpsfix->when <= dive->when + dive->duration.seconds)) {
-						copy_gps_location(gpsfix,dive);
+					if (time_during_dive_with_offset(dive, gpsfix->when, 0)) {
+						if (verbose)
+							qDebug() << "gpsfix is during the dive, pick that one";
+						copy_gps_location(gpsfix, dive);
 						changed++;
 						tracer = j;
 						break;
@@ -54,23 +83,58 @@ static bool merge_locations_into_dives(void)
 						/*
 						 * If it is not, check if there are more position fixes in SAME_GROUP range
 						 */
-						if ((nextgpsfix = get_gps_location(j+1,&gps_location_table)) &&
-						    dive_within_time_range (dive, nextgpsfix->when, SAME_GROUP)) {
-							/*
-							 * If distance from gpsfix to end of dive is shorter than distance between
-							 * gpsfix and nextgpsfix, gpsfix is the good one. Asign, mark and end loop.
-							 * If not, simply fail and nextgpsfix will be evaluated in next iteration.
-							 */
-							if ((dive->when + dive->duration.seconds - gpsfix->when) < (nextgpsfix->when - gpsfix->when)) {
-								copy_gps_location(gpsfix,dive);
+						if ((nextgpsfix = get_dive_from_table(j + 1, &gps_location_table)) &&
+						    time_during_dive_with_offset(dive, nextgpsfix->when, SAME_GROUP)) {
+							if (verbose)
+								qDebug() << "look at the next gps fix @" << get_dive_date_string(nextgpsfix->when);
+							/* first let's test if this one is during the dive */
+							if (time_during_dive_with_offset(dive, nextgpsfix->when, 0)) {
+								if (verbose)
+									qDebug() << "which is during the dive, pick that one";
+								copy_gps_location(nextgpsfix, dive);
+								changed++;
+								tracer = j + 1;
+								break;
+							}
+							/* we know the gps fixes are sorted; if they are both before the dive, ignore the first,
+							 * if theay are both after the dive, take the first,
+							 * if the first is before and the second is after, take the closer one */
+							if (nextgpsfix->when < dive->when) {
+								if (verbose)
+									qDebug() << "which is closer to the start of the dive, do continue with that";
+								continue;
+							} else if (gpsfix->when > dive->when + dive->duration.seconds) {
+								if (verbose)
+									qDebug() << "which is even later after the end of the dive, so pick the previous one";
+								copy_gps_location(gpsfix, dive);
+								changed++;
 								tracer = j;
 								break;
+							} else {
+								/* ok, gpsfix is before, nextgpsfix is after */
+								if (dive->when - gpsfix->when <= nextgpsfix->when - (dive->when + dive->duration.seconds)) {
+									if (verbose)
+										qDebug() << "pick the one before as it's closer to the start";
+									copy_gps_location(gpsfix, dive);
+									changed++;
+									tracer = j;
+									break;
+								} else {
+									if (verbose)
+										qDebug() << "pick the one after as it's closer to the start";
+									copy_gps_location(nextgpsfix, dive);
+									changed++;
+									tracer = j + 1;
+									break;
+								}
 							}
 						/*
 						 * If no more positions in range, the actual is the one. Asign, mark and end loop.
 						 */
 						} else {
-							copy_gps_location(gpsfix,dive);
+							if (verbose)
+								qDebug() << "which seems to be the best one for this dive, so pick it";
+							copy_gps_location(gpsfix, dive);
 							changed++;
 							tracer = j;
 							break;
@@ -126,9 +190,10 @@ bool DivelogsDeWebServices::prepare_dives_for_divelogs(const QString &tempfile,
 		FILE *f;
 		char filename[PATH_MAX];
 		int streamsize;
-		char *membuf;
+		const char *membuf;
 		xmlDoc *transformed;
 		struct zip_source *s;
+		struct membuffer mb = { 0 };
 
 		/*
 		 * Get the i'th dive in XML format so we can process it.
@@ -136,29 +201,29 @@ bool DivelogsDeWebServices::prepare_dives_for_divelogs(const QString &tempfile,
 		 */
 		if (selected && !dive->selected)
 			continue;
-		QString innerTmpFile = tempfile;
-		QString tmpSuffix = QString::number(qrand() % 9999) + ".tmp";
-		innerTmpFile.replace(".dld", tmpSuffix);
-		f = subsurface_fopen(QFile::encodeName(QDir::toNativeSeparators(innerTmpFile)), "w+");
-		if (!f) {
-			report_error(tr("cannot create temporary file: %s").toUtf8(), qt_error_string().toUtf8().data());
-			goto error_close_zip;
-		}
-		save_dive(f, dive);
-		fseek(f, 0, SEEK_END);
-		streamsize = ftell(f);
-		rewind(f);
+		/* make sure the buffer is empty and add the dive */
+		mb.len = 0;
 
-		membuf = (char *)malloc(streamsize + 1);
-		if (!membuf || (streamsize = fread(membuf, 1, streamsize, f)) == 0) {
-			report_error(tr("internal error: %s").toUtf8(), qt_error_string().toUtf8().data());
-			fclose(f);
-			free((void *)membuf);
-			goto error_close_zip;
+		struct dive_site *ds = get_dive_site_by_uuid(dive->dive_site_uuid);
+
+		if (ds) {
+			put_format(&mb, "<divelog><divesites><site uuid='%8x' name='", dive->dive_site_uuid);
+			put_quoted(&mb, ds->name, 1, 0);
+			put_format(&mb, "'");
+			if (ds->latitude.udeg || ds->longitude.udeg) {
+				put_degrees(&mb, ds->latitude, " gps='", " ");
+				put_degrees(&mb, ds->longitude, "", "'");
+			}
+			put_format(&mb, "/>\n</divesites>\n");
 		}
-		membuf[streamsize] = 0;
-		fclose(f);
-		unlink(QFile::encodeName(QDir::toNativeSeparators(innerTmpFile)));
+
+		save_one_dive_to_mb(&mb, dive);
+
+		if (ds) {
+			put_format(&mb, "</divelog>\n");
+		}
+		membuf = mb_cstring(&mb);
+		streamsize = strlen(membuf);
 		/*
 		 * Parse the memory buffer into XML document and
 		 * transform it to divelogs.de format, finally dumping
@@ -168,7 +233,6 @@ bool DivelogsDeWebServices::prepare_dives_for_divelogs(const QString &tempfile,
 		if (!doc) {
 			qWarning() << errPrefix << "could not parse back into memory the XML file we've just created!";
 			report_error(tr("internal error").toUtf8());
-			free((void *)membuf);
 			goto error_close_zip;
 		}
 		free((void *)membuf);
@@ -210,7 +274,7 @@ WebServices::WebServices(QWidget *parent, Qt::WindowFlags f) : QDialog(parent, f
 	ui.buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
 	timeout.setSingleShot(true);
 	defaultApplyText = ui.buttonBox->button(QDialogButtonBox::Apply)->text();
-	userAgent = UserSurvey::getUserAgent();
+	userAgent = getUserAgent();
 }
 
 void WebServices::hidePassword()
@@ -326,15 +390,24 @@ void SubsurfaceWebServices::buttonClicked(QAbstractButton *button)
 	ui.buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
 	switch (ui.buttonBox->buttonRole(button)) {
 	case QDialogButtonBox::ApplyRole: {
+		int i;
+		struct dive *d;
+		struct dive_site *ds;
+		bool changed = false;
 		clear_table(&gps_location_table);
 		QByteArray url = tr("Webservice").toLocal8Bit();
 		parse_xml_buffer(url.data(), downloadedData.data(), downloadedData.length(), &gps_location_table, NULL);
-
+		// make sure we mark all the dive sites that were created
+		for (i = 0; i < gps_location_table.nr; i++) {
+			d = get_dive_from_table(i, &gps_location_table);
+			ds = get_dive_site_by_uuid(d->dive_site_uuid);
+			if (ds)
+				ds->notes = strdup("SubsurfaceWebservice");
+		}
 		/* now merge the data in the gps_location table into the dive_table */
 		if (merge_locations_into_dives()) {
+			changed = true;
 			mark_divelist_changed(true);
-			MainWindow::instance()->globe()->repopulateLabels();
-			MainWindow::instance()->globe()->centerOnCurrentDive();
 			MainWindow::instance()->information()->updateDiveInfo();
 		}
 
@@ -358,6 +431,27 @@ void SubsurfaceWebServices::buttonClicked(QAbstractButton *button)
 		hide();
 		close();
 		resetState();
+		/* and now clean up and remove all the extra dive sites that were created */
+		QSet<uint32_t> usedUuids;
+		for_each_dive(i, d) {
+			if (d->dive_site_uuid)
+				usedUuids.insert(d->dive_site_uuid);
+		}
+		for_each_dive_site(i, ds) {
+			if (!usedUuids.contains(ds->uuid) && same_string(ds->notes, "SubsurfaceWebservice")) {
+				delete_dive_site(ds->uuid);
+				i--; // otherwise we skip one site
+			}
+		}
+#ifndef NO_MARBLE
+		// finally now that all the extra GPS fixes that weren't used have been deleted
+		// we can update the globe
+		if (changed) {
+			GlobeGPS::instance()->repopulateLabels();
+			GlobeGPS::instance()->centerOnDiveSite(get_dive_site_by_uuid(current_dive->dive_site_uuid));
+		}
+#endif
+
 	} break;
 	case QDialogButtonBox::RejectRole:
 		if (reply != NULL && reply->isOpen()) {
@@ -378,13 +472,9 @@ void SubsurfaceWebServices::buttonClicked(QAbstractButton *button)
 void SubsurfaceWebServices::startDownload()
 {
 	QUrl url("http://api.hohndel.org/api/dive/get/");
-#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
-	url.addQueryItem("login", ui.userID->text().toUpper());
-#else
 	QUrlQuery query;
 	query.addQueryItem("login", ui.userID->text().toUpper());
 	url.setQuery(query);
-#endif
 
 	QNetworkRequest request;
 	request.setUrl(url);
@@ -606,8 +696,10 @@ void DivelogsDeWebServices::prepareDivesForUpload(bool selected)
 			f.remove();
 			return;
 		}
+	} else {
+		report_error("Failed to create upload file %s\n", qPrintable(filename));
 	}
-	MainWindow::instance()->showError(get_error_string());
+	MainWindow::instance()->getNotificationWidget()->showNotification(get_error_string(), KMessageWidget::Error);
 }
 
 void DivelogsDeWebServices::uploadDives(QIODevice *dldContent)
@@ -638,7 +730,9 @@ void DivelogsDeWebServices::uploadDives(QIODevice *dldContent)
 	}
 }
 
-DivelogsDeWebServices::DivelogsDeWebServices(QWidget *parent, Qt::WindowFlags f) : WebServices(parent, f), uploadMode(false)
+DivelogsDeWebServices::DivelogsDeWebServices(QWidget *parent, Qt::WindowFlags f) : WebServices(parent, f),
+	multipart(NULL),
+	uploadMode(false)
 {
 	QSettings s;
 	ui.userID->setText(s.value("divelogde_user").toString());
@@ -702,19 +796,11 @@ void DivelogsDeWebServices::startDownload()
 	request.setRawHeader("User-Agent", userAgent.toUtf8());
 	request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
 
-#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
-	QUrl body;
-	body.addQueryItem("user", ui.userID->text());
-	body.addQueryItem("pass", ui.password->text());
-
-	reply = manager()->post(request, body.encodedQuery());
-#else
 	QUrlQuery body;
 	body.addQueryItem("user", ui.userID->text());
 	body.addQueryItem("pass", ui.password->text());
 
 	reply = manager()->post(request, body.query(QUrl::FullyEncoded).toLatin1());
-#endif
 	connect(reply, SIGNAL(finished()), this, SLOT(listDownloadFinished()));
 	connect(reply, SIGNAL(error(QNetworkReply::NetworkError)),
 		this, SLOT(downloadError(QNetworkReply::NetworkError)));
@@ -747,22 +833,12 @@ void DivelogsDeWebServices::listDownloadFinished()
 	request.setRawHeader("User-Agent", userAgent.toUtf8());
 	request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
 
-#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
-	QUrl body;
-	body.addQueryItem("user", ui.userID->text());
-	body.addQueryItem("pass", ui.password->text());
-	body.addQueryItem("ids", diveList.idList);
-
-	reply = manager()->post(request, body.encodedQuery());
-#else
 	QUrlQuery body;
 	body.addQueryItem("user", ui.userID->text());
 	body.addQueryItem("pass", ui.password->text());
 	body.addQueryItem("ids", diveList.idList);
 
 	reply = manager()->post(request, body.query(QUrl::FullyEncoded).toLatin1());
-#endif
-
 	connect(reply, SIGNAL(readyRead()), this, SLOT(saveToZipFile()));
 	connectSignalsForDownload(reply);
 }
@@ -791,9 +867,16 @@ void DivelogsDeWebServices::downloadFinished()
 	zipFile.seek(0);
 #if defined(Q_OS_UNIX) && defined(LIBZIP_VERSION_MAJOR)
 	int duppedfd = dup(zipFile.handle());
-	struct zip *zip = zip_fdopen(duppedfd, 0, &errorcode);
-	if (!zip)
-		::close(duppedfd);
+	struct zip *zip = NULL;
+	if (duppedfd >= 0) {
+		zip = zip_fdopen(duppedfd, 0, &errorcode);
+		if (!zip)
+			::close(duppedfd);
+	} else {
+		QMessageBox::critical(this, tr("Problem with download"),
+				      tr("The archive could not be opened:\n"));
+		return;
+	}
 #else
 	struct zip *zip = zip_open(QFile::encodeName(zipFile.fileName()), 0, &errorcode);
 #endif
@@ -923,3 +1006,96 @@ QNetworkReply* UserSurveyServices::sendSurvey(QString values)
 	reply = manager()->get(request);
 	return reply;
 }
+
+CloudStorageAuthenticate::CloudStorageAuthenticate(QObject *parent) :
+	QObject(parent),
+	reply(NULL)
+{
+	userAgent = getUserAgent();
+}
+
+#define CLOUDURL QString(prefs.cloud_base_url)
+#define CLOUDBACKENDSTORAGE CLOUDURL + "/storage"
+#define CLOUDBACKENDVERIFY CLOUDURL + "/verify"
+#define CLOUDBACKENDUPDATE CLOUDURL + "/update"
+
+QNetworkReply* CloudStorageAuthenticate::backend(QString email, QString password, QString pin, QString newpasswd)
+{
+	QString payload(email + " " + password);
+	QUrl requestUrl;
+	if (pin == "" && newpasswd == "") {
+		requestUrl = QUrl(CLOUDBACKENDSTORAGE);
+	} else if (newpasswd != "") {
+		requestUrl = QUrl(CLOUDBACKENDUPDATE);
+		payload += " " + newpasswd;
+	} else {
+		requestUrl = QUrl(CLOUDBACKENDVERIFY);
+		payload += " " + pin;
+	}
+	QNetworkRequest *request = new QNetworkRequest(requestUrl);
+	request->setRawHeader("Accept", "text/xml, text/plain");
+	request->setRawHeader("User-Agent", userAgent.toUtf8());
+	request->setHeader(QNetworkRequest::ContentTypeHeader, "text/plain");
+	reply = WebServices::manager()->post(*request, qPrintable(payload));
+	connect(reply, SIGNAL(finished()), this, SLOT(uploadFinished()));
+	connect(reply, SIGNAL(sslErrors(QList<QSslError>)), this, SLOT(sslErrors(QList<QSslError>)));
+	connect(reply, SIGNAL(error(QNetworkReply::NetworkError)), this,
+		SLOT(uploadError(QNetworkReply::NetworkError)));
+	return reply;
+}
+
+void CloudStorageAuthenticate::uploadFinished()
+{
+	static QString myLastError;
+
+	QString cloudAuthReply(reply->readAll());
+	qDebug() << "Completed connection with cloud storage backend, response" << cloudAuthReply;
+	if (cloudAuthReply == "[VERIFIED]" || cloudAuthReply == "[OK]") {
+		prefs.cloud_verification_status = CS_VERIFIED;
+		NotificationWidget *nw = MainWindow::instance()->getNotificationWidget();
+		if (nw->getNotificationText() == myLastError)
+			nw->hideNotification();
+		myLastError.clear();
+	} else if (cloudAuthReply == "[VERIFY]") {
+		prefs.cloud_verification_status = CS_NEED_TO_VERIFY;
+	} else if (cloudAuthReply == "[PASSWDCHANGED]") {
+		free(prefs.cloud_storage_password);
+		prefs.cloud_storage_password = prefs.cloud_storage_newpassword;
+		prefs.cloud_storage_newpassword = NULL;
+		emit passwordChangeSuccessful();
+		return;
+	} else {
+		prefs.cloud_verification_status = CS_INCORRECT_USER_PASSWD;
+		myLastError = cloudAuthReply;
+		report_error("%s", qPrintable(cloudAuthReply));
+		MainWindow::instance()->getNotificationWidget()->showNotification(get_error_string(), KMessageWidget::Error);
+	}
+	emit finishedAuthenticate();
+}
+
+void CloudStorageAuthenticate::uploadError(QNetworkReply::NetworkError error)
+{
+	qDebug() << "Received error response from cloud storage backend:" << reply->errorString();
+}
+
+void CloudStorageAuthenticate::sslErrors(QList<QSslError> errorList)
+{
+	if (verbose) {
+		qDebug() << "Received error response trying to set up https connection with cloud storage backend:";
+		Q_FOREACH (QSslError err, errorList) {
+			qDebug() << err.errorString();
+		}
+	}
+	QSslConfiguration conf = reply->sslConfiguration();
+	QSslCertificate cert = conf.peerCertificate();
+	QByteArray hexDigest = cert.digest().toHex();
+	if (reply->url().toString().contains(prefs.cloud_base_url) &&
+	    hexDigest == "13ff44c62996cfa5cd69d6810675490e") {
+		if (verbose)
+			qDebug() << "Overriding SSL check as I recognize the certificate digest" << hexDigest;
+		reply->ignoreSslErrors();
+	} else {
+		if (verbose)
+			qDebug() << "got invalid SSL certificate with hex digest" << hexDigest;
+	}
+}
